@@ -17,13 +17,14 @@
 """
 akida_models main command-line interface
 
-This package entry-point allows akida models to be instantiated and saved at a
+This package entry-point allows Keras models of Akida model zoo to be instantiated and saved at a
 specified location.
 
 """
 
 import argparse
 import os
+import sys
 
 from .utk_face.model_vgg import vgg_utk_face
 from .kws.model_ds_cnn import ds_cnn_kws
@@ -33,16 +34,16 @@ from .imagenet.model_akidanet import akidanet_imagenet
 from .imagenet.model_akidanet_edge import akidanet_edge_imagenet
 from .imagenet.model_akidanet18 import akidanet18_imagenet
 from .detection.model_yolo import yolo_base
-from .dvs.model_convtiny_handy import convtiny_dvs_handy
-from .dvs.model_convtiny_gesture import convtiny_dvs_gesture
+from .dvs.model_convtiny import convtiny_dvs
 from .mnist.model_gxnor import gxnor_mnist
 from .transformers.model_vit import vit_ti16, bc_vit_ti16
 from .transformers.model_deit import deit_ti16, bc_deit_ti16
 from .portrait128.model_akida_unet import akida_unet_portrait128
 from .centernet.model_centernet import centernet_base
 from .macs import display_macs
-from .utils import load_weights, save_weights, load_model
+from .model_io import load_weights, save_weights, load_model
 from .unfuse_sepconv_layers import unfuse_sepconv2d
+import onnx
 
 
 def save_model_weights(model_path, weights_path):
@@ -142,6 +143,19 @@ def add_common_args(parser, default_classes, default_alpha=None, default_img_siz
                             help="The base weights to load on model.")
 
 
+def summary_model(model_path):
+    """ CLI entry point to display a model architecture from h5/onnx/fbz file.
+
+    Args:
+        model_path (str): Path to the model.
+    """
+    model = load_model(model_path)
+    if isinstance(model, onnx.ModelProto):
+        print(onnx.helper.printable_graph(model.graph), file=sys.stdout)
+    else:
+        model.summary()
+
+
 def main():
     """ CLI entry point.
 
@@ -152,7 +166,7 @@ def main():
     """
     parser = argparse.ArgumentParser()
     sp = parser.add_subparsers(dest="action")
-    c_parser = sp.add_parser("create", help="Create an akida Keras model")
+    c_parser = sp.add_parser("create", help="Create an Akida Keras model")
     c_parser.add_argument("-s",
                           "--save_model",
                           type=str,
@@ -161,16 +175,13 @@ def main():
     csp = c_parser.add_subparsers(dest="model",
                                   help="The type of model to be instantiated")
     csp.add_parser("vgg_utk_face", help="A VGG-like UTKFace model")
-    csp.add_parser("convtiny_dvs_handy", help="A Convtiny DVS handy model")
 
     dvsh_parser = csp.add_parser("convtiny_dvs_handy",
-                                 help="A Convtiny DVS model for akida")
+                                 help="A Convtiny DVS handy model")
     add_common_args(dvsh_parser, default_classes=9)
 
-    csp.add_parser("convtiny_dvs_gesture", help="A Convtiny DVS gesture model")
-
     dvsg_parser = csp.add_parser("convtiny_dvs_gesture",
-                                 help="A Convtiny DVS model for akida")
+                                 help="A Convtiny DVS gesture model")
     add_common_args(dvsg_parser, default_classes=10)
 
     csp.add_parser(
@@ -179,16 +190,21 @@ def main():
         " Spotting example")
 
     mb_parser = csp.add_parser("mobilenet_imagenet",
-                               help="A MobileNet V1 model for akida")
+                               help="A MobileNet V1 model for Akida")
     add_common_args(mb_parser, default_classes=1000, default_alpha=1.0, default_img_size=224)
 
     an_parser = csp.add_parser("akidanet_imagenet", help="An AkidaNet model")
     add_common_args(an_parser, default_classes=1000, default_img_size=224, default_alpha=1.0)
 
     ane_parser = csp.add_parser("akidanet_edge_imagenet",
-                                help="An AkidaNet model modified for akida \
+                                help="An AkidaNet model modified for Akida \
                                 edge learning")
     add_common_args(ane_parser, default_classes=1000, bw_required=True)
+    ane_parser.add_argument("-bl",
+                            "--base_layer",
+                            type=str,
+                            default="classifier",
+                            help="Last layer of the base model.")
 
     an18_parser = csp.add_parser("akidanet18_imagenet", help="An AkidaNet18 model")
     add_common_args(an18_parser, default_classes=1000, default_img_size=224)
@@ -211,7 +227,7 @@ def main():
                            default=5,
                            help="The number of anchor boxes")
 
-    csp.add_parser("gxnor_mnist", help="A GXNOR MNIST model for akida")
+    csp.add_parser("gxnor_mnist", help="A GXNOR MNIST model for Akida")
 
     akun_parser = csp.add_parser("akida_unet_portrait128",
                                  help="An Akida U-Net model for Portrait128 segmentation.")
@@ -219,8 +235,10 @@ def main():
                              help="The base AkidaNet weights to use for the encoder.")
 
     cn_parser = csp.add_parser("centernet", help="A CenterNet model.")
-    add_common_args(cn_parser, default_classes=2, default_alpha=0.5, default_img_size=224,
-                    bw_required=False)
+    add_common_args(cn_parser, default_classes=20, bw_required=False)
+    cn_parser.add_argument("-i", "--image_size", type=int, default=384,
+                           choices=[224, 384],
+                           help="The square input image size")
 
     vit_parser = csp.add_parser("vit_ti16", help="A ViT model")
     add_transformers_args(vit_parser, add_customs=True)
@@ -229,6 +247,8 @@ def main():
 
     bc_vit_parser = csp.add_parser("bc_vit_ti16", help="A ViT, brainchip customized model")
     add_transformers_args(bc_vit_parser)
+    bc_vit_parser.add_argument("-b", "--blocks", type=int, default=12,
+                               help="Number of transformer blocks")
 
     deit_parser = csp.add_parser("deit_ti16", help="A DeiT model")
     add_transformers_args(deit_parser, add_customs=True)
@@ -244,6 +264,8 @@ def main():
     bc_deit_parser.add_argument("-b", "--blocks", type=int, default=12,
                                 help="Number of transformer blocks")
 
+    csp.add_parser("vit_urbansound", help="An audio ViT, brainchip customized model")
+
     macs_parser = sp.add_parser("macs", help="Get MACS for a Keras model")
     macs_parser.add_argument("-m", "--model", type=str)
     macs_parser.add_argument("-v", "--verbose", action="store_true")
@@ -267,16 +289,20 @@ def main():
                           help="Model to apply the weights.")
     w_parser.add_argument("-w", "--weights_path", type=str, required=True,
                           help="Npz file that contains the weights to apply.")
-
+    # summary model
+    sum_parser = sp.add_parser(
+        "summary",
+        help="Display a model architecture from h5/onnx/fbz file.")
+    sum_parser.add_argument("-m", "--model", type=str, required=True, help="Model to display.")
     args = parser.parse_args()
     if args.action == "create":
         # Instantiate the wished model
         if args.model == "vgg_utk_face":
             model = vgg_utk_face()
         elif args.model == "convtiny_dvs_handy":
-            model = convtiny_dvs_handy(classes=args.classes)
+            model = convtiny_dvs(input_shape=(120, 160, 2), classes=args.classes)
         elif args.model == "convtiny_dvs_gesture":
-            model = convtiny_dvs_gesture(classes=args.classes)
+            model = convtiny_dvs(input_shape=(64, 64, 10), classes=args.classes)
         elif args.model == "ds_cnn_kws":
             model = ds_cnn_kws()
         elif args.model == "pointnet_plus_modelnet40":
@@ -294,7 +320,8 @@ def main():
                                       classes=args.classes)
         elif args.model == "akidanet_edge_imagenet":
             model = akidanet_edge_imagenet(base_model=args.base_weights,
-                                           classes=args.classes)
+                                           classes=args.classes,
+                                           base_layer=args.base_layer)
         elif args.model == "akidanet18_imagenet":
             input_shape = (args.image_size, args.image_size, 3)
             model = akidanet18_imagenet(input_shape, classes=args.classes)
@@ -306,7 +333,7 @@ def main():
                 model.load_weights(args.base_weights, by_name=True)
         elif args.model == "centernet":
             model = centernet_base(input_shape=(args.image_size, args.image_size, 3),
-                                   classes=args.classes, alpha=args.alpha)
+                                   classes=args.classes)
             if args.base_weights is not None:
                 model.load_weights(args.base_weights, by_name=True)
         elif args.model == "gxnor_mnist":
@@ -315,6 +342,9 @@ def main():
             model = akida_unet_portrait128()
             if args.base_weights is not None:
                 model.load_weights(args.base_weights, by_name=True)
+        elif args.model == "vit_urbansound":
+            # Create a base ViT model with 10 classes and 1 transformer block
+            model = bc_vit_ti16(input_shape=(224, 224, 1), classes=10, num_blocks=1)
         else:
             # Handle transformer models
             if args.model == "vit_ti16":
@@ -326,7 +356,7 @@ def main():
                                  softmax=args.softmax,
                                  act=args.act)
             elif args.model == "bc_vit_ti16":
-                model = bc_vit_ti16(classes=args.classes)
+                model = bc_vit_ti16(classes=args.classes, num_blocks=args.blocks)
             elif args.model == "deit_ti16":
                 input_shape = (args.image_size, args.image_size, 3)
                 model = deit_ti16(input_shape=input_shape,
@@ -377,3 +407,5 @@ def main():
         # Save unfused model
         unfused_model.save(model_path, include_optimizer=False)
         print(f"{model.name} has been unfused and saved under {model_path}.")
+    elif args.action == "summary":
+        summary_model(args.model)
