@@ -25,8 +25,11 @@
 This module provides a method to generate YOLO anchors from dataset annotations.
 """
 
+__all__ = ["generate_anchors"]
+
 import random
 import numpy as np
+from .data_utils import Coord
 
 
 def _iou(ann, centroids):
@@ -83,7 +86,7 @@ def _run_kmeans(ann_dims, anchor_num):
             return centroids
 
         # calculate new centroids
-        centroid_sums = np.zeros((anchor_num, anchor_dim), np.float)
+        centroid_sums = np.zeros((anchor_num, anchor_dim), float)
         for i in range(ann_num):
             centroid_sums[assignments[i]] += ann_dims[i]
         for j in range(anchor_num):
@@ -92,12 +95,12 @@ def _run_kmeans(ann_dims, anchor_num):
         prev_assignments = assignments.copy()
 
 
-def generate_anchors(annotations_data, num_anchors=5, grid_size=(7, 7)):
+def generate_anchors(dataset, num_anchors=5, grid_size=(7, 7)):
     """ Creates anchors by clustering dimensions of the ground truth boxes
     from the training dataset.
 
     Args:
-        annotations_data (dict): dictionnary of preprocessed VOC data
+        dataset (tf.Dataset): dataset used to generate anchors
         num_anchors (int, optional): number of anchors
         grid_size (tuple, optional): size of the YOLO grid
 
@@ -106,14 +109,19 @@ def generate_anchors(annotations_data, num_anchors=5, grid_size=(7, 7)):
     """
     annotation_dims = []
 
-    # run k_mean to find the anchors
-    for item in annotations_data:
-        cell_w = item['image_shape'][0] / grid_size[0]
-        cell_h = item['image_shape'][1] / grid_size[1]
+    for item in dataset:
+        image_height = item['image'].shape[0]
+        image_width = item['image'].shape[1]
 
-        for box in item['boxes']:
-            relative_w = float(box['x2'] - box['x1']) / cell_w
-            relative_h = float(box['y2'] - box['y1']) / cell_h
+        cell_w = image_width / grid_size[1]
+        cell_h = image_height / grid_size[0]
+
+        bboxes = item['objects']['bbox']
+        for i in range(bboxes.shape[0]):
+            box = bboxes[i].numpy()
+            # Tensorflow box (ymin/height, xmin/width, ymax/height, xmax/width)
+            relative_w = (image_width * float(box[Coord.x2] - box[Coord.x1])) / cell_w
+            relative_h = (image_height * float(box[Coord.y2] - box[Coord.y1])) / cell_h
             annotation_dims.append(tuple(map(float, (relative_w, relative_h))))
 
     annotation_dims = np.array(annotation_dims)
