@@ -18,34 +18,45 @@
 Module used to compute mAP scores for YOLO classification.
 """
 
+__all__ = ["MapEvaluation"]
+
 import keras
 import numpy as np
+import tensorflow as tf
 
-from keras.utils import io_utils
-from .processing import load_image, preprocess_image, decode_output
+from collections import defaultdict
+from keras.src.utils import io_utils
+from .processing import decode_output, desize_bboxes, get_affine_transform, preprocess_image
 from .box_utils import compute_overlap
+from .data_utils import Coord
+from tqdm import tqdm
 
 
 class MapEvaluation(keras.callbacks.Callback):
     """ Evaluate a given dataset using a given model.
         Code originally from https://github.com/fizyr/keras-retinanet.
+        Note that mAP is computed for IoU thresholds from 0.5 to 0.95
+        with a step size of 0.05.
 
         Args:
             model (keras.Model): model to evaluate.
             val_data (dict): dictionary containing validation data as obtained
              using `preprocess_widerface.py` module
+            num_valid (int): the length of the validation dataset
             labels (list): list of labels as strings
             anchors (list): list of anchors boxes
             period (int, optional): periodicity the precision is printed,
-             defaults to once per epoch.
-            obj_threshold (float, optional): confidence threshold for a box
-            nms_threshold (float, optional): non-maximal supression threshold
+                defaults to once per epoch. Defaults to 1.
+            obj_threshold (float, optional): confidence threshold for a box. Defaults to 0.5.
+            nms_threshold (float, optional): non-maximal supression threshold. Defaults to 0.5.
             max_box_per_image (int, optional): maximum number of detections per
-             image
+                image, Defaults to 10.
+            preserve_aspect_ratio (bool, optional): Whether aspect ratio is preserved
+                during resizing. Defaults to False.
             is_keras_model (bool, optional): indicated if the model is a Keras
-             model (True) or an Akida model (False)
+                model (True) or an Akida model (False). Defaults to True.
             decode_output_fn (Callable, optional): function to decode model's outputs.
-             Defaults to :func:`decode_output` (yolo decode output function).
+                Defaults to :func:`decode_output` (yolo decode output function).
 
         Returns:
             A dict mapping class names to mAP scores.
@@ -54,19 +65,21 @@ class MapEvaluation(keras.callbacks.Callback):
     def __init__(self,
                  model,
                  val_data,
+                 num_valid,
                  labels,
                  anchors,
                  period=1,
                  obj_threshold=0.5,
                  nms_threshold=0.5,
                  max_box_per_image=10,
+                 preserve_aspect_ratio=False,
                  is_keras_model=True,
                  decode_output_fn=decode_output):
 
         super().__init__()
         self._model = model
         self._data = val_data
-        self._data_len = len(val_data)
+        self._data_len = num_valid
         self._labels = labels
         self._num_classes = len(labels)
         self._anchors = anchors
@@ -74,6 +87,7 @@ class MapEvaluation(keras.callbacks.Callback):
         self._obj_threshold = obj_threshold
         self._nms_threshold = nms_threshold
         self._max_box_per_image = max_box_per_image
+        self._preserve_aspect_ratio = preserve_aspect_ratio
         self._is_keras_model = is_keras_model
         self._decode_output = decode_output_fn
 
@@ -91,39 +105,65 @@ class MapEvaluation(keras.callbacks.Callback):
         epoch += 1
         if self._period != 0 and (epoch % self._period == 0 or
                                   epoch == self.params.get('epochs', -1)):
-            _map, average_precisions = self.evaluate_map()
+            _map_dict, average_precisions = self.evaluate_map()
+            mean_map = sum(_map_dict.values()) / len(_map_dict)
             io_utils.print_msg("")
+            io_utils.print_msg('mAP 50: {:.4f}'.format(_map_dict[0.5]))
+            io_utils.print_msg('mAP 75: {:.4f}\n'.format(_map_dict[0.75]))
             for label, average_precision in average_precisions.items():
                 io_utils.print_msg(self._labels[label] + ' {:.4f}'.format(average_precision))
-            io_utils.print_msg('mAP: {:.4f}\n'.format(_map))
-            logs.update({'map': _map})
+            io_utils.print_msg('mAP: {:.4f}\n'.format(mean_map))
+            logs.update({'map': mean_map})
 
     def evaluate_map(self):
-        """ Evaluates current mAP score on the model.
+        """ Evaluates current mAP score on the model. mAP is computed for IoU
+        thresholds from 0.5 to 0.95 with a step size of 0.05
 
         Returns:
-            tuple: global mAP score and dictionnary of label and mAP for each
-            class.
+            tuple: a dictionnary containing mAP for each threshold and a dictionnary of label
+            containing mAP for each class.
         """
-        average_precisions = self._calc_avg_precisions()
-        _map = sum(average_precisions.values()) / len(average_precisions)
+        # predictions, overlaps and 10 IoU thresholds
+        self._pbar = tqdm(total=self._data_len + self._num_classes + 10, leave=False)
 
-        return _map, average_precisions
+        all_detections, all_annotations = self._get_predictions()
+        all_overlaps = self._compute_all_overlaps(all_detections, all_annotations)
 
-    def _load_annotations(self, i):
-        annots = []
+        # Thresholds from 0.5 to 0.95 with a step size of 0.05
+        iou_thresholds = np.linspace(0.5, 0.95, num=10)
+        total_iterations = len(iou_thresholds)
+        mean_avgs = defaultdict(float)
+        map_dict = {}
 
-        for obj in self._data[i]['boxes']:
-            annot = [
-                obj['x1'], obj['y1'], obj['x2'], obj['y2'],
-                self._labels.index(obj['label'])
-            ]
-            annots += [annot]
+        for th in iou_thresholds:
+            self._pbar.set_description(f"Computing average precisions th = {th:.2f}")
+            average_precisions = self._calc_avg_precisions(all_detections, all_annotations,
+                                                           all_overlaps, th)
+            _map = sum(average_precisions.values()) / len(average_precisions)
+            map_dict[th] = _map
 
-        return np.array(annots)
+            for label, ap in average_precisions.items():
+                mean_avgs[label] += ap / total_iterations
+            self._pbar.update(1)
+        self._pbar.close()
+        tf.keras.backend.clear_session()
 
-    def _calc_avg_precisions(self):
+        return map_dict, mean_avgs
 
+    def _load_annotations(self, data):
+        objects = data['objects']
+        h, w, _ = data['image'].shape
+        bbox = objects['bbox'].numpy()
+        labels = objects['label'].numpy()
+
+        x1 = (bbox[:, Coord.x1] * w).astype(int)
+        y1 = (bbox[:, Coord.y1] * h).astype(int)
+        x2 = (bbox[:, Coord.x2] * w).astype(int)
+        y2 = (bbox[:, Coord.y2] * h).astype(int)
+
+        return np.column_stack([x1, y1, x2, y2, labels])
+
+    def _get_predictions(self):
         # gather all detections and annotations
         all_detections = [[None
                            for _ in range(self._num_classes)]
@@ -131,24 +171,36 @@ class MapEvaluation(keras.callbacks.Callback):
         all_annotations = [[None
                             for _ in range(self._num_classes)]
                            for _ in range(self._data_len)]
+        self._pbar.set_description("Getting predictions")
 
-        for i in range(self._data_len):
-            raw_image = load_image(self._data[i]['image_path'])
+        for i, data in enumerate(self._data):
+            raw_image = data['image']
             raw_height, raw_width, _ = raw_image.shape
+            input_shape = self._model.input_shape[1:] if self._is_keras_model \
+                else self._model.input_shape
+
+            affine_transform = None
+            if self._preserve_aspect_ratio:
+                center = np.array([raw_width / 2., raw_height / 2.], dtype=np.float32)
+                affine_transform = get_affine_transform(center, [raw_width, raw_height],
+                                                        [input_shape[1],
+                                                        input_shape[0]])
+
+            image = preprocess_image(raw_image.numpy(), input_shape, affine_transform)
 
             if self._is_keras_model:
-                image = preprocess_image(raw_image, self._model.input_shape[1:])
                 input_image = image[np.newaxis, :]
-                output = self._model.predict(input_image)[0]
+                output = self._model.predict(input_image, verbose=0)[0]
             else:
-                image = preprocess_image(raw_image,
-                                         self._model.layers[0].input_dims)
                 input_image = image[np.newaxis, :].astype(np.uint8)
                 potentials = self._model.predict(input_image)[0]
 
-                h, w, _ = potentials.shape
-                output = potentials.reshape(
-                    (h, w, len(self._anchors), 4 + 1 + self._num_classes))
+                if self._anchors:
+                    h, w, _ = potentials.shape
+                    output = potentials.reshape(
+                        (h, w, len(self._anchors), 4 + 1 + self._num_classes))
+                else:
+                    output = potentials
 
             pred_boxes = self._decode_output(output, self._anchors, self._num_classes,
                                              self._obj_threshold, self._nms_threshold)
@@ -157,10 +209,9 @@ class MapEvaluation(keras.callbacks.Callback):
             pred_labels = np.array([box.get_label() for box in pred_boxes])
 
             if len(pred_boxes) > 0:
-                pred_boxes = np.array([[
-                    box.x1 * raw_width, box.y1 * raw_height, box.x2 * raw_width,
-                    box.y2 * raw_height, box.score
-                ] for box in pred_boxes])
+                pred_boxes = desize_bboxes(pred_boxes, score, raw_height, raw_width,
+                                           input_shape[0], input_shape[1],
+                                           self._preserve_aspect_ratio)
             else:
                 pred_boxes = np.array([[]])
 
@@ -180,13 +231,32 @@ class MapEvaluation(keras.callbacks.Callback):
             for label in range(self._num_classes):
                 all_detections[i][label] = pred_boxes[pred_labels == label, :]
 
-            annotations = self._load_annotations(i)
+            annotations = self._load_annotations(data)
 
             # copy ground truth to all_annotations
             for label in range(self._num_classes):
                 all_annotations[i][label] = annotations[annotations[:, 4] ==
                                                         label, :4].copy()
+            self._pbar.update(1)
 
+        return all_detections, all_annotations
+
+    def _compute_all_overlaps(self, all_detections, all_annotations):
+        all_overlaps = {}
+        self._pbar.set_description("Computing overlaps")
+
+        for label in range(self._num_classes):
+            for i in range(self._data_len):
+                detections = all_detections[i][label]
+                annotations = all_annotations[i][label]
+                overlaps = compute_overlap(detections, annotations,
+                                           mode="outer_product", box_format="xyxy")
+                all_overlaps[(i, label)] = overlaps
+            self._pbar.update(1)
+
+        return all_overlaps
+
+    def _calc_avg_precisions(self, all_detections, all_annotations, all_overlaps, iou_threshold):
         # compute mAP by comparing all detections and all annotations
         average_precisions = {}
 
@@ -200,9 +270,10 @@ class MapEvaluation(keras.callbacks.Callback):
                 detections = all_detections[i][label]
                 annotations = all_annotations[i][label]
                 num_annotations += annotations.shape[0]
+                overlaps = all_overlaps[(i, label)]
                 detected_annotations = []
 
-                for d in detections:
+                for idx, d in enumerate(detections):
                     scores = np.append(scores, d[4])
 
                     if annotations.shape[0] == 0:
@@ -210,12 +281,11 @@ class MapEvaluation(keras.callbacks.Callback):
                         true_positives = np.append(true_positives, 0)
                         continue
 
-                    overlaps = compute_overlap(np.expand_dims(d, axis=0), annotations,
-                                               mode="outer_product", box_format="xyxy")
-                    assigned_annotation = np.argmax(overlaps, axis=1)
-                    max_overlap = overlaps[0, assigned_annotation]
+                    assigned_annotation = np.argmax(overlaps, axis=1)[idx]
+                    max_overlap = overlaps[idx, assigned_annotation]
 
-                    if max_overlap >= 0.5 and assigned_annotation not in detected_annotations:
+                    if (max_overlap >= iou_threshold and
+                            assigned_annotation not in detected_annotations):
                         false_positives = np.append(false_positives, 0)
                         true_positives = np.append(true_positives, 1)
                         detected_annotations.append(assigned_annotation)
