@@ -23,14 +23,16 @@ import argparse
 import numpy as np
 import tensorflow as tf
 
-from keras.utils.np_utils import to_categorical
+from keras.utils import to_categorical
 
-from cnn2snn import load_quantized_model, convert
+from cnn2snn import convert
 
 from ..cyclic_lr import CyclicLR
 from ..training import (get_training_parser, compile_model, evaluate_model, evaluate_akida_model,
                         print_history_stats, RestoreBest, save_model)
+from ..extract import extract_samples
 from ..utils import fetch_file
+from ..model_io import load_model
 
 
 def get_data(dataset, batch_size):
@@ -48,12 +50,12 @@ def get_data(dataset, batch_size):
         samples.
     """
     # Load pre-processed dataset
-    train_file = fetch_file(os.path.join('http://data.brainchip.com/dataset-mirror',
+    train_file = fetch_file(os.path.join('https://data.brainchip.com/dataset-mirror',
                                          dataset, dataset + '_preprocessed_train.npz'),
                             fname=dataset + '_preprocessed_train.npz',
                             cache_subdir=os.path.join('datasets', dataset))
 
-    test_file = fetch_file(os.path.join('http://data.brainchip.com/dataset-mirror',
+    test_file = fetch_file(os.path.join('https://data.brainchip.com/dataset-mirror',
                                         dataset, dataset + '_preprocessed_test.npz'),
                            fname=dataset + '_preprocessed_test.npz',
                            cache_subdir=os.path.join('datasets', dataset))
@@ -131,11 +133,12 @@ def main():
 
     parsers = get_training_parser(batch_size=32,
                                   tune=True,
+                                  extract=True,
                                   global_parser=global_parser)
     args = parsers[0].parse_args()
 
     # Load the source model
-    model = load_quantized_model(args.model)
+    model = load_model(args.model)
 
     # Compile model
     learning_rate = 0.001
@@ -158,13 +161,16 @@ def main():
     elif args.action == "eval":
         # Evaluate model accuracy
         if args.akida:
-            model_ak = convert(model, input_is_image=False)
+            model_ak = convert(model)
             preds, labels = evaluate_akida_model(model_ak, test_ds, 'softmax')
             accuracy = (np.squeeze(np.argmax(preds, 1)) == np.argmax(labels,
                                                                      1)).mean()
             print(f"Akida accuracy: {accuracy}")
         else:
             evaluate_model(model, test_ds)
+    elif args.action == 'extract':
+        # Extract samples from dataset
+        extract_samples(args.savefile, train_ds, args.batch_size)
 
 
 if __name__ == "__main__":
