@@ -33,7 +33,6 @@ import keras
 from keras.callbacks import LearningRateScheduler
 from keras import Sequential
 from keras.optimizers import SGD, Adam
-from tensorflow_addons.optimizers import LAMB
 from keras.layers import Input
 from keras.models import clone_model
 
@@ -42,13 +41,15 @@ from cnn2snn import convert
 from quantizeml.layers import AddPositionEmbs, ClassToken
 
 from .preprocessing import preprocess_image, DATA_AUGMENTATION
-from ..training import (get_training_parser, calibrate_model, freeze_model_before,
-                        print_history_stats, RestoreBest, save_model)
-from ..calibrable import calibration_required
+from ..training import (get_training_parser, freeze_model_before, print_history_stats, RestoreBest,
+                        save_model)
+from ..extract import extract_samples
 from ..transformers.model_vit import apply_embedding_weights
 from ..distiller import DeitDistiller
 from ..param_scheduler import get_cosine_lr_scheduler
-from ..utils import get_tensorboard_callback, load_model
+from ..optimizers import LAMB
+from ..utils import get_tensorboard_callback
+from ..model_io import load_model
 
 
 def get_imagenet_dataset(data_path, training, image_size, batch_size, data_aug=True, one_hot=False):
@@ -57,7 +58,7 @@ def get_imagenet_dataset(data_path, training, image_size, batch_size, data_aug=T
     Args:
         data_path (str): path to the folder containing ImageNet tar files
         training (bool): True to retrieve training data, False for validation
-        image_size (int): desired image size
+        image_size (tuple): desired image size
         batch_size (int): the batch size
         data_aug (bool, optional): True to apply data augmentation (only train). Defaults to True.
         one_hot (bool, optional): whether to one hot labels or not. Defaults to False.
@@ -200,20 +201,20 @@ def rescale_legacy(base_model, input_size):
 
     Args:
         base_model (keras.Model): the model to rescale
-        input_size (int): desired model input size
+        input_size (tuple): desired model input size
 
     Returns:
         keras.Model: the rescaled model
     """
     # Create the desired input
-    input_shape = (input_size, input_size, base_model.input.shape[-1])
+    input_shape = (input_size[0], input_size[1], base_model.input.shape[-1])
     new_input = Input(input_shape)
 
     # Workaround to force the input shape update that is not working for
     # functional models: the input_tensors parameter is ignored as described in
     # https://github.com/tensorflow/tensorflow/issues/40617.
     if not isinstance(base_model, Sequential):
-        base_model.layers[0]._batch_input_shape = (None, input_size, input_size,
+        base_model.layers[0]._batch_input_shape = (None, input_size[0], input_size[1],
                                                    base_model.input.shape[-1])
         new_input = None
 
@@ -231,13 +232,13 @@ def rescale_vit(base_model, input_size):
 
     Args:
         base_model (keras.Model): the model to rescale
-        input_size (int): desired model input size
+        input_size (tuple): desired model input size
 
     Returns:
         keras.Model: the rescaled model
     """
     # 1. Create the desired input layer:
-    input_shape = (None, input_size, input_size, base_model.input.shape[-1])
+    input_shape = (None, input_size[0], input_size[1], base_model.input.shape[-1])
     num_tokens = sum(isinstance(ly, ClassToken) for ly in base_model.layers)
 
     # 2. Clone the model by modification of dict_config:
@@ -252,12 +253,15 @@ def rescale_vit(base_model, input_size):
             layer_config["config"]["batch_input_shape"] = input_shape
         # 2.2. Recover total of patches in both directions
         if "Conv2D" in layer_config["class_name"]:
-            x_patch = input_size // layer_config["config"]["kernel_size"][0]
-            y_patch = input_size // layer_config["config"]["kernel_size"][1]
+            x_patch = input_size[0] // layer_config["config"]["kernel_size"][0]
+            y_patch = input_size[1] // layer_config["config"]["kernel_size"][1]
         # 2.3. Change values in reshape process
         elif layer_config["class_name"] == "Reshape":
             layer_config["config"]["target_shape"] = list(layer_config["config"]["target_shape"])
             layer_config["config"]["target_shape"][0] = x_patch * y_patch
+        # Remove serialized build_config, otherwise from_config call that follows will build
+        # variables with the initial shape instead of the target shape
+        layer_config.pop('build_config', None)
 
     # 2.5. Recompile model with new configuration
     clone = base_model.from_config(clone_config)
@@ -278,11 +282,17 @@ def rescale(base_model, input_size):
 
     Args:
         base_model (keras.Model): the model to rescale
-        input_size (int): desired model input size
+        input_size (list): desired model input size
 
     Returns:
         keras.Model: the rescaled model
     """
+    assert len(input_size) <= 2, f"Cannot rescale to {input_size}, only 2D array are supported."
+    if len(input_size) == 1:
+        input_size = (input_size[0], input_size[0])
+    else:
+        input_size = tuple(input_size)
+
     is_vit = any(isinstance(layer, AddPositionEmbs) for layer in base_model.layers)
     if is_vit:
         return rescale_vit(base_model, input_size)
@@ -319,15 +329,10 @@ def train(model,
             ['exp_decay', 'cosine_decay', 'cosine_sched'] for exponential decay, cosine decay and
             cosine oscillation respectively. Defaults to 'exp_decay'.
     """
-    # 1. Check model validity
-    if calibration_required(model):
-        str_action = "tune" if tune else "train"
-        raise ValueError(f"{model.name} model must be calibrated before a {str_action}.")
-
-    # 2. Define training callbacks
+    # 1. Define training callbacks
     callbacks = []
 
-    # 2.1 Learning rate scheduler
+    # 1.1 Learning rate scheduler
     if lr_policy == 'exp_decay':
         LR_START = learning_rate
         LR_END = 1e-4
@@ -384,15 +389,15 @@ def train(model,
         raise ValueError(f"Unsupported learning rate policy '{lr_policy}'.")
     callbacks.append(lr_scheduler)
 
-    # 2.2 Model checkpoints (save best model and retrieve it when training is complete)
+    # 1.2 Model checkpoints (save best model and retrieve it when training is complete)
     restore_model = RestoreBest(model)
     callbacks.append(restore_model)
 
-    # 2.3 Tensorboard logs
+    # 1.3 Tensorboard logs
     tensorboard = get_tensorboard_callback(out_dir, prefix='imagenet')
     callbacks.append(tensorboard)
 
-    # 3. Train
+    # 2. Train
     history = model.fit(train_dataset,
                         steps_per_epoch=train_steps,
                         epochs=num_epochs,
@@ -405,13 +410,16 @@ def train(model,
 
 def main():
     """ Entry point for script and CLI usage.
+
+    Note: Download the ImageNet training and validation dataset tar files from
+    [ImageNetwebsite](https://image-net.org/index.php)
     """
     global_parser = argparse.ArgumentParser(add_help=False)
     global_parser.add_argument(
         "-d",
-        "--data_dir",
+        "--data",
         type=str,
-        default='/hdd/datasets/imagenet/',
+        required=True,
         help="The directory containing the ImageNet data.")
     global_parser.add_argument("-o",
                                "--out_dir",
@@ -420,7 +428,7 @@ def main():
                                help="The output directory (logs, checkpoints).")
 
     parsers = get_training_parser(batch_size=128,
-                                  calibrate=True,
+                                  extract=True,
                                   freeze_before=True,
                                   tune=True,
                                   global_parser=global_parser)
@@ -497,14 +505,18 @@ def main():
                              help="Number of samples to use (for Akida)")
 
     subparsers = parsers[-1]
-    rescale_parser = subparsers.add_parser("rescale",
-                                           help="Rescale a model.",
-                                           parents=[global_parser])
+    rescale_parser = subparsers.add_parser("rescale", help="Rescale a model.")
+    rescale_parser.add_argument("-m",
+                                "--model",
+                                type=str,
+                                required=True,
+                                help="Model to load")
     rescale_parser.add_argument("-i",
                                 "--input_size",
                                 type=int,
+                                nargs='+',
                                 required=True,
-                                help="The square input image size")
+                                help="The input image size")
     rescale_parser.add_argument("-s",
                                 "--savemodel",
                                 type=str,
@@ -515,6 +527,7 @@ def main():
 
     # Load the source model
     model = load_model(args.model)
+    im_size = model.input_shape[1:3]
 
     # Try to load the teacher model, used after to train with Knowledge Distillation
     # Hyperparameters takes from https://arxiv.org/pdf/2012.12877.pdf
@@ -535,20 +548,20 @@ def main():
                             distiller_type=getattr(args, "distiller_type", "none"))
 
     # Load validation data
-    if args.action != 'rescale':
-        im_size = model.input_shape[1]
-        val_ds, val_steps = get_imagenet_dataset(args.data_dir, False, im_size,
+    if args.action in ['train', 'tune', 'eval']:
+        val_ds, val_steps = get_imagenet_dataset(args.data, False, im_size,
                                                  args.batch_size, data_aug=False, one_hot=one_hot)
+    # Load training data
+    if args.action in ['train', 'tune', 'extract']:
+        data_aug = args.data_aug if args.action in ['train', 'tune'] else False
+        train_ds, train_steps = get_imagenet_dataset(args.data, True, im_size,
+                                                     args.batch_size, data_aug, one_hot=one_hot)
 
     # Disable QuantizeML assertions
     os.environ["ASSERT_ENABLED"] = "0"
 
     # Train model
     if args.action in ['train', 'tune']:
-        # Load training data
-        train_ds, train_steps = get_imagenet_dataset(args.data_dir, True,
-                                                     im_size, args.batch_size,
-                                                     args.data_aug, one_hot=one_hot)
         tune = args.action == 'tune'
 
         learning_rate = args.learning_rate
@@ -580,14 +593,9 @@ def main():
         m = rescale(model, args.input_size)
         save_model(m, args.model, args.savemodel, args.action)
 
-    elif args.action == "calibrate":
-        # Calibrate model
-        # Load training data without data augmentation
-        train_ds, train_steps = get_imagenet_dataset(args.data_dir, True,
-                                                     im_size, args.batch_size,
-                                                     False, one_hot=one_hot)
-        calibrate_model(model, train_ds, args.num_samples, args.batch_size, args.epochs)
-        save_model(model, args.model, args.savemodel, args.action)
+    elif args.action == 'extract':
+        # Extract samples from dataset
+        extract_samples(args.savefile, train_ds, args.batch_size)
 
 
 if __name__ == "__main__":
