@@ -22,16 +22,17 @@ architectures. It can be used for multiple use cases through transfer learning.
 
 """
 
+__all__ = ["akidanet_imagenet", "akidanet_imagenet_pretrained",
+           "akidanet_faceidentification_pretrained", "akidanet_plantvillage_pretrained",
+           "akidanet_vww_pretrained"]
+
 from keras import Model, regularizers
 from keras.layers import Input, Dropout, Rescaling
 
-from cnn2snn import quantize, load_quantized_model
-
 from .imagenet_utils import obtain_input_shape
 from ..layer_blocks import conv_block, separable_conv_block, dense_block
-from ..utils import fetch_file
-
-BASE_WEIGHT_PATH = 'http://data.brainchip.com/models/AkidaV1/akidanet/'
+from ..utils import fetch_file, get_params_by_version
+from ..model_io import load_model, get_model_path
 
 
 def akidanet_imagenet(input_shape=None,
@@ -39,12 +40,12 @@ def akidanet_imagenet(input_shape=None,
                       include_top=True,
                       pooling=None,
                       classes=1000,
-                      weight_quantization=0,
-                      activ_quantization=0,
-                      input_weight_quantization=None,
                       input_scaling=(128, -1)):
     """Instantiates the AkidaNet architecture.
 
+    Note: input preprocessing is included as part of the model (as a Rescaling layer). This model
+    expects inputs to be float tensors of pixels with values in the [0, 255] range.
+
     Args:
         input_shape (tuple, optional): shape tuple. Defaults to None.
         alpha (float, optional): controls the width of the model.
@@ -69,27 +70,9 @@ def akidanet_imagenet(input_shape=None,
               model will be a 2D tensor.
         classes (int, optional): optional number of classes to classify images
             into, only to be specified if `include_top` is `True`. Defaults to 1000.
-        weight_quantization (int, optional): sets all weights in the model to have
-            a particular quantization bitwidth except for the weights in the
-            first layer.
-            Defaults to 0.
-
-            * '0' implements floating point 32-bit weights.
-            * '2' through '8' implements n-bit weights where n is from 2-8 bits.
-        activ_quantization (int, optional): sets all activations in the model to have a
-            particular activation quantization bitwidth.
-            Defaults to 0.
-
-            * '0' implements floating point 32-bit activations.
-            * '2' through '8' implements n-bit weights where n is from 2-8 bits.
-        input_weight_quantization (int, optional): sets weight quantization in the first layer.
-            Defaults to weight_quantization value.
-
-            * '0' implements floating point 32-bit weights.
-            * '2' through '8' implements n-bit weights where n is from 2-8 bits.
         input_scaling (tuple, optional): scale factor and offset to apply to
             inputs. Defaults to (128, -1). Note that following Akida convention,
-            the scale factor is an integer used as a divider.
+            the scale factor is an integer used as a divisor.
 
     Returns:
         keras.Model: a Keras model for AkidaNet/ImageNet.
@@ -97,14 +80,13 @@ def akidanet_imagenet(input_shape=None,
     Raises:
         ValueError: in case of invalid input shape.
     """
-    # check if overrides have been provided and override
-    if input_weight_quantization is None:
-        input_weight_quantization = weight_quantization
-
     # Define weight regularization, will apply to the convolutional layers and
     # to all pointwise weights of separable convolutional layers.
     weight_regularizer = regularizers.l2(4e-5)
 
+    # Model version management
+    fused, post_relu_gap, relu_activation = get_params_by_version(relu_v2='ReLU7.5')
+
     # Determine proper input shape and default size.
     if input_shape is None:
         default_size = 224
@@ -138,7 +120,7 @@ def akidanet_imagenet(input_shape=None,
                    use_bias=False,
                    strides=2,
                    add_batchnorm=True,
-                   add_activation=True,
+                   relu_activation=relu_activation,
                    kernel_regularizer=weight_regularizer)
 
     x = conv_block(x,
@@ -148,7 +130,7 @@ def akidanet_imagenet(input_shape=None,
                    padding='same',
                    use_bias=False,
                    add_batchnorm=True,
-                   add_activation=True,
+                   relu_activation=relu_activation,
                    kernel_regularizer=weight_regularizer)
 
     x = conv_block(x,
@@ -159,7 +141,7 @@ def akidanet_imagenet(input_shape=None,
                    strides=2,
                    use_bias=False,
                    add_batchnorm=True,
-                   add_activation=True,
+                   relu_activation=relu_activation,
                    kernel_regularizer=weight_regularizer)
 
     x = conv_block(x,
@@ -169,7 +151,7 @@ def akidanet_imagenet(input_shape=None,
                    padding='same',
                    use_bias=False,
                    add_batchnorm=True,
-                   add_activation=True,
+                   relu_activation=relu_activation,
                    kernel_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -180,7 +162,8 @@ def akidanet_imagenet(input_shape=None,
                              strides=2,
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -190,7 +173,8 @@ def akidanet_imagenet(input_shape=None,
                              padding='same',
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -201,7 +185,8 @@ def akidanet_imagenet(input_shape=None,
                              strides=2,
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -211,7 +196,8 @@ def akidanet_imagenet(input_shape=None,
                              padding='same',
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -221,7 +207,8 @@ def akidanet_imagenet(input_shape=None,
                              padding='same',
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -231,7 +218,8 @@ def akidanet_imagenet(input_shape=None,
                              padding='same',
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -241,7 +229,8 @@ def akidanet_imagenet(input_shape=None,
                              padding='same',
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -251,7 +240,8 @@ def akidanet_imagenet(input_shape=None,
                              padding='same',
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -262,7 +252,8 @@ def akidanet_imagenet(input_shape=None,
                              strides=2,
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     # Last separable layer with global pooling
@@ -275,7 +266,9 @@ def akidanet_imagenet(input_shape=None,
                              pooling=layer_pooling,
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
+                             post_relu_gap=post_relu_gap,
                              pointwise_regularizer=weight_regularizer)
 
     if include_top:
@@ -284,23 +277,14 @@ def akidanet_imagenet(input_shape=None,
                         classes,
                         name='classifier',
                         add_batchnorm=False,
-                        add_activation=False,
+                        relu_activation=False,
                         kernel_regularizer=weight_regularizer)
 
     # Create model.
-    model = Model(img_input,
-                  x,
-                  name='akidanet_%0.2f_%s_%s' %
-                  (alpha, input_shape[0], classes))
-
-    if ((weight_quantization != 0) or (activ_quantization != 0) or
-            (input_weight_quantization != 0)):
-        return quantize(model, weight_quantization, activ_quantization,
-                        input_weight_quantization)
-    return model
+    return Model(img_input, x, name='akidanet_%0.2f_%s_%s' % (alpha, input_shape[0], classes))
 
 
-def akidanet_imagenet_pretrained(alpha=1.0):
+def akidanet_imagenet_pretrained(alpha=1.0, quantized=True):
     """
     Helper method to retrieve an `akidanet_imagenet` model that was trained on
     ImageNet dataset.
@@ -308,80 +292,153 @@ def akidanet_imagenet_pretrained(alpha=1.0):
     Args:
         alpha (float, optional): width of the model, allowed values in [0.25,
             0.5, 1]. Defaults to 1.0.
+        quantized (bool, optional): a boolean indicating whether the model should be loaded
+            quantized or not. Defaults to True.
 
     Returns:
         keras.Model: a Keras Model instance.
 
     """
     if alpha == 1.0:
-        model_name = 'akidanet_imagenet_224_iq8_wq4_aq4.h5'
-        file_hash = '359e0ff05abbb26fe215830ca467ef40761767c0d77f8c743c6d6e4fffa7a925'
+        if quantized:
+            model_name_v1 = 'akidanet_imagenet_224_iq8_wq4_aq4.h5'
+            file_hash_v1 = '359e0ff05abbb26fe215830ca467ef40761767c0d77f8c743c6d6e4fffa7a925'
+            model_name_v2 = 'akidanet_imagenet_224_alpha_1_i8_w4_a4.h5'
+            file_hash_v2 = '5c6d3009a0a62139696234b86ddbad5186e889e2c351f727e2b96c3926816a5c'
+        else:
+            model_name_v1 = 'akidanet_imagenet_224.h5'
+            file_hash_v1 = '9208233d21c3251777b1f700b7c2c51491590133f97f81f04348ab131679b00f'
+            model_name_v2 = 'akidanet_imagenet_224_alpha_1.h5'
+            file_hash_v2 = 'fc4aaf81a8aec8792709ae0fa10aac2dcb9517202a6f919e049c4db96f272339'
     elif alpha == 0.5:
-        model_name = 'akidanet_imagenet_224_alpha_50_iq8_wq4_aq4.h5'
-        file_hash = '1d9493115d43625f2644f8265f71b8487c4019047fc331e892a233a3d6520371'
+        if quantized:
+            model_name_v1 = 'akidanet_imagenet_224_alpha_50_iq8_wq4_aq4.h5'
+            file_hash_v1 = '1d9493115d43625f2644f8265f71b8487c4019047fc331e892a233a3d6520371'
+            model_name_v2 = 'akidanet_imagenet_224_alpha_0.5_i8_w4_a4.h5'
+            file_hash_v2 = '1f2acdabe7fca379f2e8e13df2c080eec18090fe155869f2351fc15c8ab1829b'
+        else:
+            model_name_v1 = 'akidanet_imagenet_224_alpha_50.h5'
+            file_hash_v1 = '61f2883a6b798f922a5c0411296219a85f25581d7571f65546557b46066f058f'
+            model_name_v2 = 'akidanet_imagenet_224_alpha_0.5.h5'
+            file_hash_v2 = '6d202e9477d89192abe069e3a87105e0db8fe4a6a1a152afc933b687dd5418b8'
     elif alpha == 0.25:
-        model_name = 'akidanet_imagenet_224_alpha_25_iq8_wq4_aq4.h5'
-        file_hash = '9146d6228d859d8b8db1c1b7a795471096e5a49ee4ff5656eabc6be749d42f5e'
+        if quantized:
+            model_name_v1 = 'akidanet_imagenet_224_alpha_25_iq8_wq4_aq4.h5'
+            file_hash_v1 = '9146d6228d859d8b8db1c1b7a795471096e5a49ee4ff5656eabc6be749d42f5e'
+            model_name_v2 = 'akidanet_imagenet_224_alpha_0.25_i8_w4_a4.h5'
+            file_hash_v2 = '3aad95e41263c393b6a3fc80c8788615b0d53b938b7f7702b52fdbb93e76b7f1'
+        else:
+            model_name_v1 = 'akidanet_imagenet_224_alpha_25.h5'
+            file_hash_v1 = '23d96a1c73397a5c2060808f3843d99fa3cd96b5b1af36e03b0d039be393c001'
+            model_name_v2 = 'akidanet_imagenet_224_alpha_0.25.h5'
+            file_hash_v2 = 'bb2b39fc2e41009547eb267fd8b6c66d7a5a16e72ffed4a641ec82f50fb4370f'
     else:
         raise ValueError(
             f"Requested model with alpha={alpha} is not available.")
 
-    model_path = fetch_file(BASE_WEIGHT_PATH + model_name,
+    model_path, model_name, file_hash = get_model_path("akidanet", model_name_v1, file_hash_v1,
+                                                       model_name_v2, file_hash_v2)
+    model_path = fetch_file(model_path,
                             fname=model_name,
                             file_hash=file_hash,
                             cache_subdir='models')
-    return load_quantized_model(model_path)
+    return load_model(model_path)
 
 
-def akidanet_faceidentification_pretrained():
+def akidanet_faceidentification_pretrained(quantized=True):
     """
     Helper method to retrieve an `akidanet_imagenet` model that was trained on
     CASIA Webface dataset and that performs face identification.
 
+    Args:
+        quantized (bool, optional): a boolean indicating whether the model should be loaded
+            quantized or not. Defaults to True.
+
     Returns:
         keras.Model: a Keras Model instance.
 
     """
-    model_name = 'akidanet_faceidentification_iq8_wq4_aq4.h5'
-    file_hash = '117df1118124c5d2cc0cf867dcc10fa1dfbc3978c1bfe0dcba727ac369e48765'
-    model_path = fetch_file(BASE_WEIGHT_PATH + model_name,
+    if quantized:
+        model_name_v1 = 'akidanet_faceidentification_iq8_wq4_aq4.h5'
+        file_hash_v1 = 'b287f86155c51dc73053f7e5f3e58be1beb4a35d543dd817a63a782ffaf5bff1'
+        model_name_v2 = 'akidanet_faceidentification_i8_w4_a4.h5'
+        file_hash_v2 = 'd38af7f3bb24d1e8f0e0471d8afb80ae1c8cb3589031e153abac120434bde8e7'
+    else:
+        model_name_v1 = 'akidanet_faceidentification.h5'
+        file_hash_v1 = '998b6cdce5adbdb0b7c3e9f8eb1011c3582e05f0c159e473d226a9a94c4309a1'
+        model_name_v2 = 'akidanet_faceidentification.h5'
+        file_hash_v2 = 'f7b65fbf520f51b3c0c6d8cb7fcc78391536f200a45dbc8b9bd16262ebd82b18'
+
+    model_path, model_name, file_hash = get_model_path("akidanet", model_name_v1, file_hash_v1,
+                                                       model_name_v2, file_hash_v2)
+    model_path = fetch_file(model_path,
                             fname=model_name,
                             file_hash=file_hash,
                             cache_subdir='models')
-    return load_quantized_model(model_path)
+    return load_model(model_path)
 
 
-def akidanet_plantvillage_pretrained():
+def akidanet_plantvillage_pretrained(quantized=True):
     """
     Helper method to retrieve an `akidanet_imagenet` model that was trained on
     PlantVillage dataset.
 
+    Args:
+        quantized (bool, optional): a boolean indicating whether the model should be loaded
+            quantized or not. Defaults to True.
+
     Returns:
         keras.Model: a Keras Model instance.
 
     """
-    model_name = 'akidanet_plantvillage_iq8_wq4_aq4.h5'
-    file_hash = '1400910c774fd78a5e6ea227ff28cb28e79ecec0909a378068cd9f40ddaf4e0a'
-    model_path = fetch_file(BASE_WEIGHT_PATH + model_name,
+    if quantized:
+        model_name_v1 = 'akidanet_plantvillage_iq8_wq4_aq4.h5'
+        file_hash_v1 = '1400910c774fd78a5e6ea227ff28cb28e79ecec0909a378068cd9f40ddaf4e0a'
+        model_name_v2 = 'akidanet_plantvillage_i8_w4_a4.h5'
+        file_hash_v2 = '75a852761c73be14c3f2b424cf5703bf9f7a57cb9de0a917b71cf9b9b3e355da'
+    else:
+        model_name_v1 = None
+        file_hash_v1 = None
+        model_name_v2 = 'akidanet_plantvillage.h5'
+        file_hash_v2 = '625145dec17e286beb61b9f683dc2f518752e5f9203d3b69b27c07d802528dbd'
+
+    model_path, model_name, file_hash = get_model_path("akidanet", model_name_v1, file_hash_v1,
+                                                       model_name_v2, file_hash_v2)
+    model_path = fetch_file(model_path,
                             fname=model_name,
                             file_hash=file_hash,
                             cache_subdir='models')
-    return load_quantized_model(model_path)
+    return load_model(model_path)
 
 
-def akidanet_vww_pretrained():
+def akidanet_vww_pretrained(quantized=True):
     """
     Helper method to retrieve an `akidanet_imagenet` model that was trained on
     VWW dataset.
 
+    Args:
+        quantized (bool, optional): a boolean indicating whether the model should be loaded
+            quantized or not. Defaults to True.
+
     Returns:
         keras.Model: a Keras Model instance.
 
     """
-    model_name = 'akidanet_vww_iq8_wq4_aq4.h5'
-    file_hash = 'cd130d90ed736447b6244dc1228e708b9dab20af0d2bf57b9a49df4362467ea8'
-    model_path = fetch_file(BASE_WEIGHT_PATH + model_name,
+    if quantized:
+        model_name_v1 = 'akidanet_vww_iq8_wq4_aq4.h5'
+        file_hash_v1 = 'cd130d90ed736447b6244dc1228e708b9dab20af0d2bf57b9a49df4362467ea8'
+        model_name_v2 = 'akidanet_vww_i8_w4_a4.h5'
+        file_hash_v2 = 'a54cb42906ed5eb253bac223123a1bbe7e14e4e188b4d877feb195759f85d9b2'
+    else:
+        model_name_v1 = 'akidanet_vww.h5'
+        file_hash_v1 = '00e03f13226cd622ad92bdb3402c4b4399a69875f2dde6ccadfb235ad6994d78'
+        model_name_v2 = 'akidanet_vww.h5'
+        file_hash_v2 = '635cbb3cfa486f44d0fca7ca66dbc794b87b67d7d77ab32e06780ecb278c2c58'
+
+    model_path, model_name, file_hash = get_model_path("akidanet", model_name_v1, file_hash_v1,
+                                                       model_name_v2, file_hash_v2)
+    model_path = fetch_file(model_path,
                             fname=model_name,
                             file_hash=file_hash,
                             cache_subdir='models')
-    return load_quantized_model(model_path)
+    return load_model(model_path)
