@@ -21,17 +21,16 @@ AkidaNet18 architecture is inspired both from AkidaNet and ResNet18: same depth
 ResNet18 but without skip connections and using SeparableConvolution layers.
 
 """
+
+__all__ = ["akidanet18_imagenet", "akidanet18_imagenet_pretrained"]
+
 from keras import Model, regularizers
 from keras.layers import Input, Rescaling
 
-from cnn2snn import quantize, load_quantized_model
-
 from .imagenet_utils import obtain_input_shape
 from ..layer_blocks import conv_block, separable_conv_block, dense_block
 from ..utils import fetch_file
-
-
-BASE_WEIGHT_PATH = 'http://data.brainchip.com/models/AkidaV1/akidanet18/'
+from ..model_io import load_model, get_model_path
 
 
 def akidanet18_imagenet(input_shape=None,
@@ -40,12 +39,12 @@ def akidanet18_imagenet(input_shape=None,
                         classes=1000,
                         depths=(4, 4, 4, 4),
                         dimensions=(64, 128, 256, 512),
-                        weight_quantization=0,
-                        activ_quantization=0,
-                        input_weight_quantization=None,
                         input_scaling=(128, -1)):
     """Instantiates the AkidaNet18 architecture.
 
+    Note: input preprocessing is included as part of the model (as a Rescaling layer). This model
+    expects inputs to be float tensors of pixels with values in the [0, 255] range.
+
     Args:
         input_shape (tuple, optional): shape tuple. Defaults to None.
         include_top (bool, optional): whether to include the fully-connected
@@ -66,27 +65,9 @@ def akidanet18_imagenet(input_shape=None,
         dimensions (tuple, optional): number of filters in each stage on the model. The length of
             the tuple must be equal to the length of the `depth` tuple. Defaults to
             (64, 128, 256, 512).
-        weight_quantization (int, optional): sets all weights in the model to have
-            a particular quantization bitwidth except for the weights in the
-            first layer.
-            Defaults to 0.
-
-            * '0' implements floating point 32-bit weights.
-            * '2' through '8' implements n-bit weights where n is from 2-8 bits.
-        activ_quantization (int, optional): sets all activations in the model to have a
-            particular activation quantization bitwidth.
-            Defaults to 0.
-
-            * '0' implements floating point 32-bit activations.
-            * '2' through '8' implements n-bit weights where n is from 2-8 bits.
-        input_weight_quantization (int, optional): sets weight quantization in the first layer.
-            Defaults to weight_quantization value.
-
-            * '0' implements floating point 32-bit weights.
-            * '2' through '8' implements n-bit weights where n is from 2-8 bits.
         input_scaling (tuple, optional): scale factor and offset to apply to
             inputs. Defaults to (128, -1). Note that following Akida convention,
-            the scale factor is an integer used as a divider.
+            the scale factor is an integer used as a divisor.
 
     Returns:
         keras.Model: a Keras model for AkidaNet/ImageNet.
@@ -94,10 +75,6 @@ def akidanet18_imagenet(input_shape=None,
     Raises:
         ValueError: in case of invalid input shape or mismatching `depth` and `dimensions`.
     """
-    # Check if overrides have been provided and override
-    if input_weight_quantization is None:
-        input_weight_quantization = weight_quantization
-
     # Sanity checks
     stages = len(depths)
     if len(dimensions) != stages:
@@ -142,7 +119,7 @@ def akidanet18_imagenet(input_shape=None,
                    use_bias=False,
                    strides=4,
                    add_batchnorm=True,
-                   add_activation=True,
+                   relu_activation='ReLU7.5',
                    kernel_regularizer=weight_regularizer)
 
     # Define the stages
@@ -154,7 +131,7 @@ def akidanet18_imagenet(input_shape=None,
             kwarg = {"kernel_regularizer": weight_regularizer}
         else:
             current_block = separable_conv_block
-            kwarg = {"pointwise_regularizer": weight_regularizer}
+            kwarg = {"pointwise_regularizer": weight_regularizer, "fused": False}
 
         strides = 2 if stage > 0 else 1
         for i in range(depths[stage]):
@@ -177,7 +154,8 @@ def akidanet18_imagenet(input_shape=None,
                               use_bias=False,
                               pooling=pool,
                               add_batchnorm=True,
-                              add_activation=True,
+                              relu_activation='ReLU7.5',
+                              post_relu_gap=True,
                               **kwarg)
 
     # Classification layer
@@ -185,33 +163,39 @@ def akidanet18_imagenet(input_shape=None,
         x = dense_block(x,
                         classes,
                         add_batchnorm=False,
-                        add_activation=False,
+                        relu_activation=False,
                         kernel_initializer="he_normal",
                         name='classifier',
                         kernel_regularizer=weight_regularizer)
 
     # Create model
-    model = Model(img_input, x, name='akidanet18_%s_%s' % (input_shape[0], classes))
-
-    if ((weight_quantization != 0) or (activ_quantization != 0) or
-            (input_weight_quantization != 0)):
-        return quantize(model, weight_quantization, activ_quantization,
-                        input_weight_quantization)
-    return model
+    return Model(img_input, x, name='akidanet18_%s_%s' % (input_shape[0], classes))
 
 
-def akidanet18_imagenet_pretrained():
+def akidanet18_imagenet_pretrained(quantized=True):
     """
     Helper method to retrieve an `akidanet18_imagenet` model that was trained on ImageNet dataset.
 
+    Args:
+        quantized (bool, optional): a boolean indicating whether the model should be loaded
+            quantized or not. Defaults to True.
+
     Returns:
         keras.Model: a Keras Model instance.
 
     """
-    model_name = 'akidanet18_imagenet_224_iq8_wq4_aq4.h5'
-    file_hash = 'd3a0a3f37b5d3a5998981705e772b40670fdab0dd68e5894dc8af717bbaae136'
-    model_path = fetch_file(BASE_WEIGHT_PATH + model_name,
+    # Note: cannot be converted to v1 so we should ultimately removed v1 support and only keep v2
+    if quantized:
+        model_name_v2 = 'akidanet18_imagenet_224_i8_w8_a8.h5'
+        file_hash_v2 = '3ee6309b6a8e5aad570fc22d750e9d1e8db82d9a250e9d8f09e009a815db7a3c'
+    else:
+        model_name_v2 = 'akidanet18_imagenet_224.h5'
+        file_hash_v2 = 'e52cf2a20b786a488b75f371b8b89f0c95a383dafc1331eea51b2356552d9704'
+
+    model_path, model_name, file_hash = get_model_path("akidanet18", model_name_v2=model_name_v2,
+                                                       file_hash_v2=file_hash_v2)
+    model_path = fetch_file(model_path,
                             fname=model_name,
                             file_hash=file_hash,
                             cache_subdir='models')
-    return load_quantized_model(model_path)
+    return load_model(model_path)
