@@ -27,16 +27,15 @@ compatible for Akida with:
     - different initialization options.
 """
 
+__all__ = ["mobilenet_imagenet", "mobilenet_imagenet_pretrained"]
+
 from keras import Model, regularizers
 from keras.layers import Input, Dropout, Rescaling
 
-from cnn2snn import load_quantized_model, quantize
-
 from .imagenet_utils import obtain_input_shape
 from ..layer_blocks import conv_block, separable_conv_block, dense_block
-from ..utils import fetch_file
-
-BASE_WEIGHT_PATH = 'http://data.brainchip.com/models/AkidaV1/mobilenet/'
+from ..utils import fetch_file, get_params_by_version
+from ..model_io import load_model, get_model_path
 
 
 def mobilenet_imagenet(input_shape=None,
@@ -46,12 +45,12 @@ def mobilenet_imagenet(input_shape=None,
                        pooling=None,
                        classes=1000,
                        use_stride2=True,
-                       weight_quantization=0,
-                       activ_quantization=0,
-                       input_weight_quantization=None,
                        input_scaling=(128, -1)):
     """Instantiates the MobileNet architecture.
 
+    Note: input preprocessing is included as part of the model (as a Rescaling layer). This model
+    expects inputs to be float tensors of pixels with values in the [0, 255] range.
+
     Args:
         input_shape (tuple, optional): shape tuple. Defaults to None.
         alpha (float, optional): controls the width of the model.
@@ -79,27 +78,9 @@ def mobilenet_imagenet(input_shape=None,
             into, only to be specified if `include_top` is `True`. Defaults to 1000.
         use_stride2 (bool, optional): replace max pooling operations by stride 2
             convolutions in layers separable 2, 4, 6 and 12. Defaults to True.
-        weight_quantization (int, optional): sets all weights in the model to have
-            a particular quantization bitwidth except for the weights in the
-            first layer.
-            Defaults to 0.
-
-            * '0' implements floating point 32-bit weights.
-            * '2' through '8' implements n-bit weights where n is from 2-8 bits.
-        activ_quantization (int, optional): sets all activations in the model to have a
-            particular activation quantization bitwidth.
-            Defaults to 0.
-
-            * '0' implements floating point 32-bit activations.
-            * '2' through '8' implements n-bit weights where n is from 2-8 bits.
-        input_weight_quantization (int, optional): sets weight quantization in the first layer.
-            Defaults to weight_quantization value.
-
-            * '0' implements floating point 32-bit weights.
-            * '2' through '8' implements n-bit weights where n is from 2-8 bits.
         input_scaling (tuple, optional): scale factor and offset to apply to
             inputs. Defaults to (128, -1). Note that following Akida convention,
-            the scale factor is an integer used as a divider.
+            the scale factor is an integer used as a divisor.
 
     Returns:
         keras.Model: a Keras model for MobileNet/ImageNet.
@@ -107,9 +88,8 @@ def mobilenet_imagenet(input_shape=None,
     Raises:
         ValueError: in case of invalid input shape.
     """
-    # check if overrides have been provided and override
-    if input_weight_quantization is None:
-        input_weight_quantization = weight_quantization
+    # Model version management
+    fused, post_relu_gap, relu_activation = get_params_by_version(relu_v2='ReLU7.5')
 
     # Define weight regularization, will apply to the first convolutional layer
     # and to all pointwise weights of separable convolutional layers.
@@ -159,7 +139,7 @@ def mobilenet_imagenet(input_shape=None,
                    use_bias=False,
                    strides=2,
                    add_batchnorm=True,
-                   add_activation=True,
+                   relu_activation=relu_activation,
                    kernel_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -169,7 +149,8 @@ def mobilenet_imagenet(input_shape=None,
                              padding='same',
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -181,7 +162,8 @@ def mobilenet_imagenet(input_shape=None,
                              strides=strides,
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -191,7 +173,8 @@ def mobilenet_imagenet(input_shape=None,
                              padding='same',
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -203,7 +186,8 @@ def mobilenet_imagenet(input_shape=None,
                              strides=strides,
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -213,7 +197,8 @@ def mobilenet_imagenet(input_shape=None,
                              padding='same',
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -225,7 +210,8 @@ def mobilenet_imagenet(input_shape=None,
                              strides=strides,
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -235,7 +221,8 @@ def mobilenet_imagenet(input_shape=None,
                              padding='same',
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -245,7 +232,8 @@ def mobilenet_imagenet(input_shape=None,
                              padding='same',
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -255,7 +243,8 @@ def mobilenet_imagenet(input_shape=None,
                              padding='same',
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -265,7 +254,8 @@ def mobilenet_imagenet(input_shape=None,
                              padding='same',
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -275,7 +265,8 @@ def mobilenet_imagenet(input_shape=None,
                              padding='same',
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     x = separable_conv_block(x,
@@ -287,7 +278,8 @@ def mobilenet_imagenet(input_shape=None,
                              strides=strides,
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
                              pointwise_regularizer=weight_regularizer)
 
     # Last separable layer with global pooling
@@ -300,7 +292,9 @@ def mobilenet_imagenet(input_shape=None,
                              pooling=layer_pooling,
                              use_bias=False,
                              add_batchnorm=True,
-                             add_activation=True,
+                             relu_activation=relu_activation,
+                             fused=fused,
+                             post_relu_gap=post_relu_gap,
                              pointwise_regularizer=weight_regularizer)
 
     if include_top:
@@ -310,48 +304,68 @@ def mobilenet_imagenet(input_shape=None,
                         name='classifier',
                         use_bias=False,
                         add_batchnorm=False,
-                        add_activation=False,
+                        relu_activation=False,
                         kernel_regularizer=weight_regularizer)
 
     # Create model.
-    model = Model(img_input,
-                  x,
-                  name='mobilenet_%0.2f_%s_%s' % (alpha, rows, classes))
-
-    if ((weight_quantization != 0) or (activ_quantization != 0) or
-            (input_weight_quantization != 0)):
-        return quantize(model, weight_quantization, activ_quantization,
-                        input_weight_quantization)
-    return model
+    return Model(img_input, x, name='mobilenet_%0.2f_%s_%s' % (alpha, rows, classes))
 
 
-def mobilenet_imagenet_pretrained(alpha=1.0):
+def mobilenet_imagenet_pretrained(alpha=1.0, quantized=True):
     """
     Helper method to retrieve a `mobilenet_imagenet` model that was trained on
     ImageNet dataset.
 
     Args:
         alpha (float): width of the model.
+        quantized (bool, optional): a boolean indicating whether the model should be loaded
+            quantized or not. Defaults to True.
 
     Returns:
         keras.Model: a Keras Model instance.
 
     """
     if alpha == 1.0:
-        model_name = 'mobilenet_imagenet_224_iq8_wq4_aq4.h5'
-        file_hash = '4e63ea329b3f2f773a0b9d6fef4e449639fda89d17e48bb7132b6ad86585c867'
+        if quantized:
+            model_name_v1 = 'mobilenet_imagenet_224_iq8_wq4_aq4.h5'
+            file_hash_v1 = '4e63ea329b3f2f773a0b9d6fef4e449639fda89d17e48bb7132b6ad86585c867'
+            model_name_v2 = 'mobilenet_imagenet_224_alpha_1_i8_w4_a4.h5'
+            file_hash_v2 = 'da329c8b6836c141a7b836875dbcabd36d8efd5b90d9719f50e2d4facea4868a'
+        else:
+            model_name_v1 = 'mobilenet_imagenet_224.h5'
+            file_hash_v1 = 'fb674f627337995f4aa372e4a270445457573fc7aae61e28c05f94da14cf2980'
+            model_name_v2 = 'mobilenet_imagenet_224_alpha_1.h5'
+            file_hash_v2 = '4305ca6e03bffaf4e39a840fb2c6108101136f572ae784bebc3bd9406a6ecd0c'
     elif alpha == 0.5:
-        model_name = 'mobilenet_imagenet_224_alpha_50_iq8_wq4_aq4.h5'
-        file_hash = '133204fe087b728adc975a3bd71c275b5dbec8974555e2fa3a6ef8727cc5c9ac'
+        if quantized:
+            model_name_v1 = 'mobilenet_imagenet_224_alpha_50_iq8_wq4_aq4.h5'
+            file_hash_v1 = 'af9a31774467de96acceeed46cee14d7864fa2cc247601beff5521e7a5e6da99'
+            model_name_v2 = 'mobilenet_imagenet_224_alpha_0.5_i8_w4_a4.h5'
+            file_hash_v2 = 'c7277a7850c777a39c90023a7c509696bc7a2cb61590d3c88841f1ee3bc14003'
+        else:
+            model_name_v1 = 'mobilenet_imagenet_224_alpha_50.h5'
+            file_hash_v1 = '1bffadb48f3fd194cdeecf4e02d91d0b3a87df103620c645f1c00f5e5cfbca9a'
+            model_name_v2 = 'mobilenet_imagenet_224_alpha_0.5.h5'
+            file_hash_v2 = '1ded4e456b59b4286329b2e8694d1d3adce9745a73a59cc4670dc0eae60ff2bc'
     elif alpha == 0.25:
-        model_name = 'mobilenet_imagenet_224_alpha_25_iq8_wq4_aq4.h5'
-        file_hash = '35ad51e662ed04b68978865ccb1c16bf024fa013146488b2a9c18c80d1efab29'
+        if quantized:
+            model_name_v1 = 'mobilenet_imagenet_224_alpha_25_iq8_wq4_aq4.h5'
+            file_hash_v1 = '35ad51e662ed04b68978865ccb1c16bf024fa013146488b2a9c18c80d1efab29'
+            model_name_v2 = 'mobilenet_imagenet_224_alpha_0.25_i8_w4_a4.h5'
+            file_hash_v2 = '29eb66a023fa808735e66f3716e3a8d6429a5d9621e6ed875de4fc8104682a14'
+        else:
+            model_name_v1 = 'mobilenet_imagenet_224_alpha_25.h5'
+            file_hash_v1 = '33c49bb4ec868558a2a9687be19a7e51756ddb9f00a032535069dcd5a727e0dd'
+            model_name_v2 = 'mobilenet_imagenet_224_alpha_0.25.h5'
+            file_hash_v2 = '5f70598352ddeb7bf2c19126d7009f569fc9cf6d52124334c4ccebe9030f3c39'
     else:
         raise ValueError(
             f"Requested model with alpha={alpha} is not available.")
 
-    model_path = fetch_file(BASE_WEIGHT_PATH + model_name,
+    model_path, model_name, file_hash = get_model_path("mobilenet", model_name_v1, file_hash_v1,
+                                                       model_name_v2, file_hash_v2)
+    model_path = fetch_file(model_path,
                             fname=model_name,
                             file_hash=file_hash,
                             cache_subdir='models')
-    return load_quantized_model(model_path)
+    return load_model(model_path)
