@@ -21,14 +21,16 @@ KWS model training script.
 import os
 import pickle
 
-from keras.utils.np_utils import to_categorical
+from keras.utils import to_categorical
 
-from cnn2snn import load_quantized_model, quantize_layer, convert
+from cnn2snn import quantize_layer, convert
 
 from ..cyclic_lr import CyclicLR
 from ..training import (get_training_parser, compile_model, evaluate_model, print_history_stats,
                         RestoreBest, save_model)
+from ..extract import extract_samples
 from ..utils import fetch_file
+from ..model_io import load_model
 
 
 def get_data():
@@ -39,7 +41,7 @@ def get_data():
     """
     # Load pre-processed dataset
     fname = fetch_file(
-        'http://data.brainchip.com/dataset-mirror/kws/kws_preprocessed_all_words_except_backward_follow_forward.pkl',
+        'https://data.brainchip.com/dataset-mirror/kws/kws_preprocessed_all_words_except_backward_follow_forward.pkl',
         fname='kws_preprocessed_all_words_except_backward_follow_forward.pkl',
         cache_subdir=os.path.join('datasets', 'kws'))
 
@@ -97,7 +99,7 @@ def train_model(model, x_train, y_train, x_val, y_val, batch_size, epochs):
 def main():
     """ Entry point for script and CLI usage.
     """
-    parsers = get_training_parser(batch_size=100, global_batch_size=False)
+    parsers = get_training_parser(batch_size=100, global_batch_size=False, extract=True)
 
     train_parser = parsers[1]
     train_parser.add_argument("-laq",
@@ -109,7 +111,7 @@ def main():
     args = parsers[0].parse_args()
 
     # Load the source model
-    model = load_quantized_model(args.model)
+    model = load_model(args.model)
 
     # If specified, change the last layer activation bitwidth
     if "last_activ_quantization" in args and args.last_activ_quantization:
@@ -136,6 +138,9 @@ def main():
             print(f"Accuracy: {accuracy}")
         else:
             evaluate_model(model, x_val, y=to_categorical(y_val))
+    elif args.action == 'extract':
+        # Extract samples from dataset
+        extract_samples(args.savefile, x_train, args.batch_size)
 
 
 if __name__ == "__main__":
