@@ -18,24 +18,17 @@
 DS-CNN model definition for KWS classification.
 """
 
-from keras import Model
-from keras.layers import (Input, Reshape, Activation, Flatten, Rescaling)
+__all__ = ["ds_cnn_kws", "ds_cnn_kws_pretrained"]
 
-from cnn2snn import load_quantized_model, quantize
+from keras import Model
+from keras.layers import Input, Reshape, Activation, Rescaling
 
 from ..layer_blocks import conv_block, separable_conv_block, dense_block
-from ..utils import fetch_file
-
-BASE_WEIGHT_PATH = 'http://data.brainchip.com/models/AkidaV1/ds_cnn/'
+from ..utils import fetch_file, get_params_by_version
+from ..model_io import load_model, get_model_path
 
 
-def ds_cnn_kws(input_shape=(49, 10, 1),
-               classes=33,
-               include_top=True,
-               weight_quantization=0,
-               activ_quantization=0,
-               input_weight_quantization=None,
-               input_scaling=(255, 0)):
+def ds_cnn_kws(input_shape=(49, 10, 1), classes=33, include_top=True, input_scaling=(255, 0)):
     """Instantiates a MobileNet-like model for the "Keyword Spotting" example.
 
     This model is based on the MobileNet architecture, mainly with fewer layers.
@@ -46,43 +39,28 @@ def ds_cnn_kws(input_shape=(49, 10, 1),
     and was created for the "Keyword Spotting" (KWS) or "Speech Commands"
     dataset.
 
+    Note: input preprocessing is included as part of the model (as a Rescaling layer). This model
+    expects inputs to be float tensors of pixels with values in the [0, 255] range.
+
     Args:
-        input_shape (tuple): input shape tuple of the model
-        classes (int): optional number of classes to classify words into, only
-            be specified if `include_top` is True.
-        include_top (bool): whether to include the fully-connected
-            layer at the top of the model.
-        weight_quantization (int): sets all weights in the model to have
-            a particular quantization bitwidth except for the weights in the
-            first layer.
-
-            * '0' implements floating point 32-bit weights.
-            * '2' through '8' implements n-bit weights where n is from 2-8 bits.
-        activ_quantization (int): sets all activations in the model to have a
-            particular activation quantization bitwidth.
-
-            * '0' implements floating point 32-bit activations.
-            * '1' through '8' implements n-bit weights where n is from 2-8 bits.
-        input_weight_quantization (int): sets weight quantization in the first
-            layer. Defaults to weight_quantization value.
-
-            * 'None' implements the same bitwidth as the other weights.
-            * '0' implements floating point 32-bit weights.
-            * '2' through '8' implements n-bit weights where n is from 2-8 bits.
+        input_shape (tuple, optional): input shape tuple of the model. Defaults to (49, 10, 1).
+        classes (int, optional): optional number of classes to classify words into, only
+            be specified if `include_top` is True. Defaults to 33.
+        include_top (bool, optional): whether to include the classification layer at the top of the
+            model. Defaults to True.
         input_scaling (tuple, optional): scale factor and offset to apply to
             inputs. Defaults to (255, 0). Note that following Akida convention,
-            the scale factor is an integer used as a divider.
+            the scale factor is an integer used as a divisor.
 
     Returns:
         keras.Model: a Keras model for MobileNet/KWS
     """
-    # Overrides input weight quantization if None
-    if input_weight_quantization is None:
-        input_weight_quantization = weight_quantization
-
     if include_top and not classes:
         raise ValueError("If 'include_top' is True, 'classes' must be set.")
 
+    # Model version management
+    fused, post_relu_gap, relu_activation = get_params_by_version()
+
     img_input = Input(shape=input_shape, name="input")
 
     if input_scaling is None:
@@ -98,7 +76,8 @@ def ds_cnn_kws(input_shape=(49, 10, 1),
                    strides=(2, 2),
                    use_bias=False,
                    name='conv_0',
-                   add_batchnorm=True)
+                   add_batchnorm=True,
+                   relu_activation=relu_activation)
 
     x = separable_conv_block(x,
                              filters=64,
@@ -106,7 +85,9 @@ def ds_cnn_kws(input_shape=(49, 10, 1),
                              padding='same',
                              use_bias=False,
                              name='separable_1',
-                             add_batchnorm=True)
+                             add_batchnorm=True,
+                             fused=fused,
+                             relu_activation=relu_activation)
 
     x = separable_conv_block(x,
                              filters=64,
@@ -114,7 +95,9 @@ def ds_cnn_kws(input_shape=(49, 10, 1),
                              padding='same',
                              use_bias=False,
                              name='separable_2',
-                             add_batchnorm=True)
+                             add_batchnorm=True,
+                             fused=fused,
+                             relu_activation=relu_activation)
 
     x = separable_conv_block(x,
                              filters=64,
@@ -122,7 +105,9 @@ def ds_cnn_kws(input_shape=(49, 10, 1),
                              padding='same',
                              use_bias=False,
                              name='separable_3',
-                             add_batchnorm=True)
+                             add_batchnorm=True,
+                             fused=fused,
+                             relu_activation=relu_activation)
 
     x = separable_conv_block(x,
                              filters=64,
@@ -131,47 +116,54 @@ def ds_cnn_kws(input_shape=(49, 10, 1),
                              use_bias=False,
                              name='separable_4',
                              pooling='global_avg',
-                             add_batchnorm=True)
-
-    shape = (1, 1, int(64))
-    x = Reshape(shape, name='reshape_1')(x)
+                             add_batchnorm=True,
+                             fused=fused,
+                             post_relu_gap=post_relu_gap,
+                             relu_activation=relu_activation)
 
     if include_top:
-        x = Flatten(name="flatten")(x)
         x = dense_block(x,
                         units=classes,
                         name='dense_5',
                         use_bias=True,
-                        add_activation=False)
+                        relu_activation=False)
         act_function = 'softmax' if classes > 1 else 'sigmoid'
         x = Activation(act_function, name=f'act_{act_function}')(x)
+    else:
+        shape = (1, 1, int(64))
+        x = Reshape(shape, name='reshape_1')(x)
 
-    model = Model(img_input, x, name='ds_cnn_kws')
-
-    if ((weight_quantization != 0) or (activ_quantization != 0) or
-            (input_weight_quantization != 0)):
-        # Converts a standard sequential Keras model to a CNN2SNN Keras
-        # quantized model, compatible for Akida conversion.
-        model = quantize(model=model,
-                         weight_quantization=weight_quantization,
-                         activ_quantization=activ_quantization,
-                         input_weight_quantization=input_weight_quantization)
-    return model
+    return Model(img_input, x, name='ds_cnn_kws')
 
 
-def ds_cnn_kws_pretrained():
+def ds_cnn_kws_pretrained(quantized=True):
     """
     Helper method to retrieve a `ds_cnn_kws` model that was trained on
     KWS dataset.
 
+    Args:
+        quantized (bool, optional): a boolean indicating whether the model should be loaded
+            quantized or not. Defaults to True.
+
     Returns:
         keras.Model: a Keras Model instance.
 
     """
-    model_name = 'ds_cnn_kws_iq8_wq4_aq4_laq1.h5'
-    file_hash = '2ba6220bb9545857c99a327ec14d2d777420c7848cb6a9b17d87e5a01951fe6f'
-    model_path = fetch_file(BASE_WEIGHT_PATH + model_name,
+    if quantized:
+        model_name_v1 = 'ds_cnn_kws_iq8_wq4_aq4_laq1.h5'
+        file_hash_v1 = '2ba6220bb9545857c99a327ec14d2d777420c7848cb6a9b17d87e5a01951fe6f'
+        model_name_v2 = 'ds_cnn_kws_edge_i8_w4_a4.h5'
+        file_hash_v2 = '30e514563be67251a89705c296a137bd79b15918bdb8d6ca7daa02a05506775b'
+    else:
+        model_name_v1 = 'ds_cnn_kws.h5'
+        file_hash_v1 = '95a51677b340ee2420015a8576a8aaf41e84138ac0334cd42080b60499b4f146'
+        model_name_v2 = 'ds_cnn_kws.h5'
+        file_hash_v2 = 'e897da2760206aad1328179c2b6861bae4f0e56a9380c78cf1287d18929a2143'
+
+    model_path, model_name, file_hash = get_model_path("ds_cnn", model_name_v1, file_hash_v1,
+                                                       model_name_v2, file_hash_v2)
+    model_path = fetch_file(model_path,
                             fname=model_name,
                             file_hash=file_hash,
                             cache_subdir='models')
-    return load_quantized_model(model_path)
+    return load_model(model_path)
