@@ -21,13 +21,14 @@ from functools import partial
 
 from keras.layers import (BatchNormalization, ReLU, Conv2D, DepthwiseConv2D, SeparableConv2D, Dense,
                           MaxPool2D, GlobalAvgPool2D, LayerNormalization, Dropout, Add,
-                          Conv2DTranspose, Reshape)
-from keras.activations import swish
+                          Conv2DTranspose, Reshape, Conv3D)
+from keras.activations import swish, gelu
 from keras.initializers import TruncatedNormal
-from tensorflow_addons.layers import GELU, GroupNormalization
+import tensorflow as tf
 
 from quantizeml.layers import (Attention, LayerMadNormalization, DepthwiseConv2DTranspose,
                                ExtractToken)
+from .utils import get_params_by_version
 
 
 def _add_pooling_layer(x, pooling_type, pool_size, padding, layer_base_name):
@@ -65,7 +66,7 @@ def conv_block(inputs,
                post_relu_gap=False,
                pool_size=(2, 2),
                add_batchnorm=False,
-               add_activation=True,
+               relu_activation='ReLU3.75',
                **kwargs):
     """Adds a convolutional layer with optional layers in the following order:
     max pooling, batch normalization, activation.
@@ -78,16 +79,17 @@ def conv_block(inputs,
             height and width of the 2D convolution kernel.
             Can be a single integer to specify the same value for
             all spatial dimensions.
-        pooling (str): add a pooling layer of type 'pooling' among the values 'max' or 'global_avg',
-            with pooling size set to pool_size. If 'None', no pooling will be added.
-        post_relu_gap (bool): when pooling is 'global_avg', indicates if the pooling comes before or
-            after ReLU activation. Defaults to False.
-        pool_size (int or tuple of 2 integers): factors by which to
-            downscale (vertical, horizontal). (2, 2) will halve the input in
-            both spatial dimension. If only one integer is specified, the same
-            window length will be used for both dimensions.
-        add_batchnorm (bool): add a BatchNormalization layer
-        add_activation (bool): add a ReLU layer
+        pooling (str, optional): add a pooling layer of type 'pooling' among the values 'max' or
+            'global_avg', with pooling size set to pool_size. If 'None', no pooling will be added.
+        post_relu_gap (bool, optional): when pooling is 'global_avg', indicates if the pooling comes
+            before or after ReLU activation. Defaults to False.
+        pool_size (int or tuple of 2 integers, optional): factors by which to downscale (vertical,
+            horizontal). (2, 2) will halve the input in both spatial dimension. If only one integer
+            is specified, the same window length will be used for both dimensions.
+        add_batchnorm (bool, optional): add a BatchNormalization layer
+        relu_activation (str, optional): the ReLU activation to add to the layer in the form 'ReLUx'
+            where 'x' is the max_value to use. Set to False to disable activation. Defaults to
+            'ReLU3.75'.
         **kwargs: arguments passed to the keras.Conv2D layer, such as
             strides, padding, use_bias, weight_regularizer, etc.
 
@@ -108,8 +110,8 @@ def conv_block(inputs,
     if add_batchnorm:
         x = BatchNormalization(name=conv_layer.name + '/BN')(x)
 
-    if add_activation:
-        x = ReLU(6.0, name=conv_layer.name + '/relu')(x)
+    if relu_activation:
+        x = act_to_layer(relu_activation, name=conv_layer.name + '/relu')(x)
 
     if post_relu_gap and pooling == 'global_avg':
         x = _add_pooling_layer(x, pooling, pool_size, conv_layer.padding, conv_layer.name)
@@ -126,7 +128,7 @@ def separable_conv_block(inputs,
                          post_relu_gap=False,
                          pool_size=(2, 2),
                          add_batchnorm=False,
-                         add_activation=True,
+                         relu_activation='ReLU3.75',
                          fused=True,
                          name=None,
                          kernel_initializer='glorot_uniform',
@@ -146,17 +148,18 @@ def separable_conv_block(inputs,
             Defaults to 1.
         padding (str, optional): padding mode for the depthwise convolution. Defaults to 'same'.
         use_bias (bool, optional): whether the layer uses a bias vector. Defaults to True.
-        pooling (str): add a pooling layer of type 'pooling' among the values 'max', or
+        pooling (str, optional): add a pooling layer of type 'pooling' among the values 'max', or
             'global_avg', with pooling size set to pool_size. If 'None', no pooling will be added.
-        post_relu_gap (bool): when pooling is 'global_avg', indicates if the pooling comes before or
-            after ReLU activation. Defaults to False.
-        pool_size (int or tuple of 2 integers): factors by which to
-            downscale (vertical, horizontal). (2, 2) will halve the input in
-            both spatial dimension. If only one integer is specified, the same
-            window length will be used for both dimensions.
-        add_batchnorm (bool): add a BatchNormalization layer
-        add_activation (bool): add a ReLU layer
-        fused (bool): If True use a SeparableConv2D layer otherwise use a
+        post_relu_gap (bool, optional): when pooling is 'global_avg', indicates if the pooling comes
+            before or after ReLU activation. Defaults to False.
+        pool_size (int or tuple of 2 integers, optional): factors by which to downscale (vertical,
+            horizontal). (2, 2) will halve the input in both spatial dimension. If only one integer
+            is specified, the same window length will be used for both dimensions.
+        add_batchnorm (bool, optional): add a BatchNormalization layer
+        relu_activation (str, optional): the ReLU activation to add to the layer in the form 'ReLUx'
+            where 'x' is the max_value to use. Set to False to disable activation. Defaults to
+            'ReLU3.75'.
+        fused (bool, optional): If True use a SeparableConv2D layer otherwise use a
             DepthwiseConv2D + Conv2D layers. Defaults to True.
         name (str, optional): name of the layer. Defaults to None.
         kernel_initializer (keras.initializer, optional): initializer for both kernels. Defaults to
@@ -189,7 +192,7 @@ def separable_conv_block(inputs,
                                            use_bias=False,
                                            depthwise_initializer=kernel_initializer,
                                            name=dw_name)
-        point_conv_layer = Conv2D(filters, (1, 1), use_bias=use_bias,
+        point_conv_layer = Conv2D(filters, (1, 1), use_bias=use_bias, padding='same',
                                   kernel_initializer=kernel_initializer,
                                   kernel_regularizer=pointwise_regularizer,
                                   name=pw_name)
@@ -205,8 +208,8 @@ def separable_conv_block(inputs,
     if add_batchnorm:
         x = BatchNormalization(name=main_layer_name + '/BN')(x)
 
-    if add_activation:
-        x = ReLU(6.0, name=main_layer_name + '/relu')(x)
+    if relu_activation:
+        x = act_to_layer(relu_activation, name=main_layer_name + '/relu')(x)
 
     if post_relu_gap and pooling == 'global_avg':
         x = _add_pooling_layer(x, pooling, pool_size, padding, main_layer_name)
@@ -216,7 +219,7 @@ def separable_conv_block(inputs,
 def dense_block(inputs,
                 units,
                 add_batchnorm=False,
-                add_activation=True,
+                relu_activation='ReLU3.75',
                 **kwargs):
     """Adds a dense layer with optional layers in the following order:
     batch normalization, activation.
@@ -224,8 +227,10 @@ def dense_block(inputs,
     Args:
         inputs (tf.Tensor): Input tensor of shape `(rows, cols, channels)`
         units (int): dimensionality of the output space
-        add_batchnorm (bool): add a BatchNormalization layer
-        add_activation (bool): add a ReLU layer
+        add_batchnorm (bool, optional): add a BatchNormalization layer
+        relu_activation (str, optional): the ReLU activation to add to the layer in the form 'ReLUx'
+            where 'x' is the max_value to use. Set to False to disable activation. Defaults to
+            'ReLU3.75'.
         **kwargs: arguments passed to the Dense layer, such as
             use_bias, kernel_initializer, weight_regularizer, etc.
 
@@ -241,8 +246,8 @@ def dense_block(inputs,
     if add_batchnorm:
         x = BatchNormalization(name=dense_layer.name + '/BN')(x)
 
-    if add_activation:
-        x = ReLU(6.0, name=dense_layer.name + '/relu')(x)
+    if relu_activation:
+        x = act_to_layer(relu_activation, name=dense_layer.name + '/relu')(x)
 
     return x
 
@@ -261,7 +266,7 @@ def act_to_layer(act, **kwargs):
         keras.layers: the activation layer class
     """
     if act == 'GeLU':
-        act_funct = GELU(**kwargs)
+        act_funct = gelu
     elif 'ReLU' in act:
         if act == 'ReLU':
             max_value = None
@@ -297,7 +302,7 @@ def norm_to_layer(norm):
     if norm == 'LN':
         norm_funct = LayerNormalization
     elif norm == 'GN1':
-        norm_funct = partial(GroupNormalization, groups=1)
+        norm_funct = partial(tf.keras.layers.GroupNormalization, groups=1)
     elif norm == 'BN':
         norm_funct = BatchNormalization
     elif norm == 'LMN':
@@ -420,7 +425,7 @@ def conv_transpose_block(inputs,
                          filters,
                          kernel_size,
                          add_batchnorm=False,
-                         add_activation=True,
+                         relu_activation='ReLU8',
                          **kwargs):
     """Adds a transposed convolutional layer with optional layers in the following order:
     batch normalization, activation.
@@ -433,7 +438,9 @@ def conv_transpose_block(inputs,
             convolution kernel. Can be a single integer to specify the same value for all spatial
             dimensions.
         add_batchnorm (bool, optional): add a BatchNormalization layer. Defaults to False.
-        add_activation (bool, optional): add a ReLU8 layer. Defaults to True.
+        relu_activation (str, optional): the ReLU activation to add to the layer in the form 'ReLUx'
+            where 'x' is the max_value to use. Set to False to disable activation. Defaults to
+            'ReLU3.75'.
         **kwargs: arguments passed to the keras.Conv2DTranspose layer, such as strides, padding,
             use_bias, weight_regularizer, etc.
 
@@ -452,8 +459,8 @@ def conv_transpose_block(inputs,
     if add_batchnorm:
         x = BatchNormalization(name=conv_trans_layer.name + '/BN')(x)
 
-    if add_activation:
-        x = ReLU(8.0, name=conv_trans_layer.name + '/relu')(x)
+    if relu_activation:
+        x = act_to_layer(relu_activation, name=conv_trans_layer.name + '/relu')(x)
 
     return x
 
@@ -465,7 +472,7 @@ def sepconv_transpose_block(inputs,
                             padding='same',
                             use_bias=True,
                             add_batchnorm=False,
-                            add_activation=True,
+                            relu_activation='ReLU3.75',
                             name=None,
                             kernel_initializer='glorot_uniform',
                             pointwise_regularizer=None):
@@ -486,7 +493,9 @@ def sepconv_transpose_block(inputs,
         padding (str, optional): padding mode for the transposed depthwise. Defaults to 'same'.
         use_bias (bool, optional): whether the layer uses a bias vectors. Defaults to True.
         add_batchnorm (bool, optional): add a BatchNormalization layer. Defaults to False.
-        add_activation (bool, optional): add a ReLU8 layer. Defaults to True.
+        relu_activation (str, optional): the ReLU activation to add to the layer in the form 'ReLUx'
+            where 'x' is the max_value to use. Set to False to disable activation. Defaults to
+            'ReLU3.75'.
         name (str, optional): name of the layer. Defaults to None.
         kernel_initializer (keras.initializer, optional): initializer for both kernels. Defaults to
             'glorot_uniform'.
@@ -522,8 +531,8 @@ def sepconv_transpose_block(inputs,
     if add_batchnorm:
         x = BatchNormalization(name=pw_layer.name + '/BN')(x)
 
-    if add_activation:
-        x = ReLU(8.0, name=pw_layer.name + '/relu')(x)
+    if relu_activation:
+        x = act_to_layer(relu_activation, name=pw_layer.name + '/relu')(x)
 
     return x
 
@@ -544,18 +553,21 @@ def yolo_head_block(x, num_boxes, classes, filters=1024):
     Notes:
         This block replaces conv layers by separable_conv, to decrease the amount of parameters.
     """
+    # Model version management
+    fused, _, relu_activation = get_params_by_version(relu_v2='ReLU7.5')
+
     x = separable_conv_block(x, filters=filters, name='1conv',
                              kernel_size=(3, 3), padding='same', use_bias=False,
-                             add_activation=True, add_batchnorm=True)
+                             relu_activation=relu_activation, add_batchnorm=True, fused=fused)
     x = separable_conv_block(x, filters=filters, name='2conv',
                              kernel_size=(3, 3), padding='same', use_bias=False,
-                             add_activation=True, add_batchnorm=True)
+                             relu_activation=relu_activation, add_batchnorm=True, fused=fused)
     x = separable_conv_block(x, filters=filters, name='3conv',
                              kernel_size=(3, 3), padding='same', use_bias=False,
-                             add_activation=True, add_batchnorm=True)
+                             relu_activation=relu_activation, add_batchnorm=True, fused=fused)
     x = separable_conv_block(x, filters=(num_boxes * (4 + 1 + classes)), name='detection_layer',
                              kernel_size=(3, 3), padding='same', use_bias=True,
-                             add_activation=False, add_batchnorm=False)
+                             relu_activation=False, add_batchnorm=False, fused=fused)
     return x
 
 
@@ -576,3 +588,129 @@ def vit_extract_feature_map(y, feat_shape, num_non_patch=1):
     y = ExtractToken(name="extract_features", token=list(range(num_non_patch, y.shape[1])))(y)
     y = Reshape(target_shape=(*feat_shape, y.shape[-1]), name="features")(y)
     return y
+
+
+def conv3d_block(inputs,
+                 filters,
+                 kernel_size,
+                 add_batchnorm=False,
+                 relu_activation='ReLU3.75',
+                 **kwargs):
+    """Adds a Conv3D layer with optional layers: batch normalization and activation.
+
+    Args:
+        inputs (tf.Tensor): input tensor
+        filters (int): the dimensionality of the output space
+        kernel_size (int or tuple): dimensions of the convolution kernel.
+        add_batchnorm (bool, optional): add a BatchNormalization layer. Defaults to False.
+        relu_activation (str, optional): the ReLU activation to add to the layer in the form 'ReLUx'
+            where 'x' is the max_value to use. Set to False to disable activation. Defaults to
+            'ReLU3.75'.
+        **kwargs: arguments passed to the keras.Conv3D layer, such as strides, use_bias, etc.
+
+    Returns:
+        tf.Tensor: output tensor of conv2D block.
+    """
+    if 'activation' in kwargs and kwargs['activation']:
+        raise ValueError("Keyword argument 'activation' in conv3d_block must be None.")
+    # If it's a temporal Conv3d with a same padding, convert it to a one sided-left padding,
+    # by adding a ZeroPadding3D layer and setting the Conv3D padding to "valid"
+    if kernel_size[0] != 1 and kwargs.get('padding', 'valid') == 'same':
+        padding_layer = tf.keras.layers.ZeroPadding3D(
+            padding=((kernel_size[0] - 1, 0), (0, 0), (0, 0)))
+        inputs = padding_layer(inputs)
+        kwargs['padding'] = 'valid'
+    conv_layer = Conv3D(filters, kernel_size, **kwargs)
+    x = conv_layer(inputs)
+
+    if add_batchnorm:
+        x = BatchNormalization(name=conv_layer.name + '/BN')(x)
+
+    if relu_activation:
+        x = act_to_layer(relu_activation, name=conv_layer.name + '/relu')(x)
+
+    return x
+
+
+def spatiotemporal_block(inputs, in_channels, med_channels, out_channels, t_kernel_size, t_stride,
+                         t_depthwise, s_depthwise, index):
+    """ Add a spatiotemporal block to the inputs.
+
+    The spatio-temporal block consists of a temporal convolution (potentially separable) followed by
+    a spatial convolution (potentially separable).
+
+    Note that the depthwise layers are implemented as Conv3D with groups=filters because TensorFlow
+    does not have a DepthwiseConv3D layer.
+
+    Args:
+        inputs (tf.Tensor): input tensor
+        in_channels (int): input channels
+        med_channels (int): middle channels (channels after the temporal conv layer)
+        out_channels (int): output channels (channels after the spatial conv layer)
+        t_kernel_size (int): the temporal kernel size
+        t_stride (int): the temporal kernel stride
+        t_depthwise (bool): whether the temporal layer is dw_separable
+        s_depthwise (bool): whether the spatial layer is dw_separable
+        index (int): index of the block
+
+    Returns:
+        tf.Tensor: output tensor of the spatiotemporal block.
+    """
+    if not t_depthwise:
+        x = conv3d_block(inputs,
+                         med_channels,
+                         (t_kernel_size, 1, 1),
+                         add_batchnorm=True,
+                         relu_activation='ReLU',
+                         strides=(t_stride, 1, 1),
+                         padding='same',
+                         use_bias=True,
+                         name=f'convt_full_{index}')
+    else:
+        # This is a DepthwiseConv3D (groups=filters)
+        x = conv3d_block(inputs,
+                         in_channels,
+                         (t_kernel_size, 1, 1),
+                         add_batchnorm=True,
+                         relu_activation='ReLU',
+                         strides=(t_stride, 1, 1),
+                         padding='same',
+                         groups=in_channels,
+                         use_bias=False,
+                         name=f'convt_dw_{index}')
+        x = conv3d_block(x,
+                         med_channels,
+                         (1, 1, 1),
+                         add_batchnorm=True,
+                         relu_activation='ReLU',
+                         name=f'convt_pw_{index}')
+
+    if not s_depthwise:
+        x = conv3d_block(x,
+                         out_channels,
+                         (1, 3, 3),
+                         add_batchnorm=True,
+                         relu_activation='ReLU',
+                         strides=(1, 2, 2),
+                         padding='same',
+                         use_bias=True,
+                         name=f'convs_full_{index}')
+    else:
+        # This is a DepthwiseConv3D (groups=filters)
+        x = conv3d_block(x,
+                         med_channels,
+                         (1, 3, 3),
+                         add_batchnorm=True,
+                         relu_activation='ReLU',
+                         strides=(1, 2, 2),
+                         padding='same',
+                         groups=med_channels,
+                         use_bias=False,
+                         name=f'convs_dw_{index}')
+        x = conv3d_block(x,
+                         out_channels,
+                         (1, 1, 1),
+                         add_batchnorm=True,
+                         relu_activation='ReLU',
+                         name=f'convs_pw_{index}')
+    return x
