@@ -2,8 +2,7 @@ import tensorflow as tf
 from tensorflow.python.framework.convert_to_constants import (
         convert_variables_to_constants_v2_as_graph)
 
-from quantizeml.models import load_model
-from cnn2snn import load_quantized_model
+from .model_io import load_model
 
 
 def get_flops(model):
@@ -58,11 +57,7 @@ def display_macs(model_path, verbose=False):
         verbose (bool): display MACS for each operation
 
     """
-    try:
-        model = load_model(model_path)
-    except ValueError:
-        model = load_quantized_model(model_path)
-
+    model = load_model(model_path)
     flops = get_flops(model)
 
     if verbose:
