@@ -17,6 +17,8 @@
 Preprocessing tools for ModelNet40 data handling.
 """
 
+__all__ = ["get_modelnet_from_file", "get_modelnet"]
+
 import os
 import glob
 import numpy as np
@@ -27,7 +29,7 @@ from .pointnet_utils import pointnet_preproc
 from ..utils import fetch_file
 
 
-def parse_dataset(data_dir, num_points=1024):
+def _parse_dataset(data_dir, num_points=1024):
     """ Parse the dataset.
 
     Args:
@@ -82,7 +84,7 @@ def parse_dataset(data_dir, num_points=1024):
     )
 
 
-def data_transformation(points, label):
+def _data_transformation(points, label):
     """ Data transformation function.
 
     Args:
@@ -149,7 +151,7 @@ def get_modelnet_from_file(num_points, filename="ModelNet40.zip"):
             data_dir = os.path.join(os.path.dirname(data_dir), short_filename)
 
         # Then read in the data and store a fixed number of points per sample
-        train_points, test_points, train_labels, test_labels, class_map = parse_dataset(
+        train_points, test_points, train_labels, test_labels, class_map = _parse_dataset(
             data_dir, num_points)
         np.savez(datafile,
                  train_points=train_points,
@@ -165,8 +167,8 @@ def get_modelnet(train_points,
                  test_points,
                  test_labels,
                  batch_size,
-                 selected_points=128,
-                 knn_points=64):
+                 selected_points=64,
+                 knn_points=32):
     """ Obtains the ModelNet dataset.
 
     Args:
@@ -175,10 +177,10 @@ def get_modelnet(train_points,
         test_points (numpy.array): test set.
         test_labels (numpy.array): test labels.
         batch_size (int): size of the batch.
-        selected_points (int): num points to process per sample. Default is 512.
+        selected_points (int): num points to process per sample. Defaults to 64.
         knn_points (int): number of points to include in each localised group.
             Must be a power of 2, and ideally an integer square (so 64, or 16
-            for a deliberately small network, or 256 for large).
+            for a deliberately small network, or 256 for large). Defaults to 32.
 
     Returns:
         tf.data.Dataset, tf.data.Dataset: train and test point with data
@@ -198,7 +200,7 @@ def get_modelnet(train_points,
 
     if len(train_points) > 0:
         train_dataset = train_dataset.shuffle(
-            len(train_points)).map(data_transformation).batch(batch_size)
+            len(train_points)).map(_data_transformation).batch(batch_size)
         train_dataset = \
             train_dataset.map(lambda points,
                               label: pointnet_preproc(points, label,
