@@ -227,3 +227,37 @@ class ParamScheduler(Callback):
         # this batch.
         for k, v in logs.items():
             self.history.setdefault(k, []).append(v)
+
+
+def get_cosine_lr_scheduler(lr_start, num_iterations, pct_start=0.3, include_beta=False):
+    """ Wrapper method to get a preconfigured ParamScheduler object.
+
+    Defines a scheduling policy for the learning rate (and optionally the Nadam decay) where two
+    phases are defined by ``pct_start``. During the phases, values will follow a
+    cosine oscillation within specific boundaries.
+
+    Args:
+        lr_start (float): initial value of the learning rate
+        num_iterations (int): number of iterations
+        pct_start (float, optional): percentage at which the second phase of the scheduler
+            will be trigged. Defaults to 0.3.
+        include_beta (bool, optional): adds a Nadam beta_1 scheduling policy. Defaults to False.
+
+    Returns:
+        ParamScheduler: the callback object
+    """
+    assert 0.0 <= pct_start <= 1.0
+    # Define learning rate scheduler
+    div = 25.
+    final_div = 25e4
+    scheds = {'learning_rate': combine_scheds([pct_start, 1 - pct_start],
+                                              [SchedCos(lr_start / div, lr_start),
+                                               SchedCos(lr_start, lr_start / final_div)])}
+    if include_beta:
+        mom_max = 0.95
+        mom_min = 0.85
+        scheds["beta_1"] = combine_scheds([pct_start, 1 - pct_start],
+                                          [SchedCos(mom_max, mom_min),
+                                           SchedCos(mom_min, mom_max)])
+
+    return ParamScheduler(scheds, total_num_iterations=num_iterations)
