@@ -27,14 +27,12 @@ from keras.preprocessing.image import DirectoryIterator
 from keras.optimizers import Adam
 from keras.layers import Activation
 from keras.callbacks import ModelCheckpoint
-from keras.utils.generic_utils import Progbar
-from quantizeml.layers import calibration
 
 
 def compile_model(model,
                   learning_rate=1e-3,
                   loss='categorical_crossentropy',
-                  metrics=None):
+                  metrics=['accuracy']):
     """ Compiles the model using Adam optimizer.
 
     Args:
@@ -43,11 +41,8 @@ def compile_model(model,
         loss (str or function, optional): the loss function. Defaults to
             'categorical_crossentropy'.
         metrics (list, optional): list of metrics to be evaluated during
-            training and testing. Defaults to None.
+            training and testing. Defaults to ['accuracy'].
     """
-    if metrics is None:
-        metrics = ['accuracy']
-
     model.compile(optimizer=Adam(learning_rate=learning_rate),
                   loss=loss,
                   metrics=metrics)
@@ -74,30 +69,6 @@ def freeze_model_before(model, freeze_before):
             raise ValueError(f"No such layer {freeze_before} in model.")
 
 
-def calibrate_model(model, x, num_samples, batch_size=None, epochs=1):
-    """Calibrates model
-
-    Args:
-        model (keras.Model): the model to calibrate
-        x (tf.Dataset, np.array or generator): train input data
-        num_samples (int): total number of samples before declaring the calibration round finished.
-        batch_size (int, optional): the batch size. Defaults to None.
-        epochs (int, optional): the number of epochs. Defaults to 1.
-    """
-    if batch_size is None:
-        steps = num_samples
-    else:
-        assert batch_size > 0, "The batch size should be strictly positive."
-        steps = np.ceil(num_samples / batch_size)
-
-    with calibration(True):
-        progbar = Progbar(epochs)
-        for epoch in range(epochs):
-            progbar.update(epoch)
-            model.predict(x=x, steps=steps, batch_size=batch_size, verbose=0)
-        progbar.update(epochs, finalize=epochs)
-
-
 def evaluate_model(model,
                    x,
                    y=None,
@@ -156,7 +127,13 @@ def evaluate_akida_model(model, x, activation):
         if not isinstance(batch, np.ndarray):
             batch = batch.numpy()
 
-        pots_batch = model.predict(batch.astype('uint8'))
+        # Retrieve input_signed value: cannot use getattr here because LayerParams does not
+        # implement a default value.
+        input_signed = 'input_signed' in dir(model.get_layer(0).parameters)
+        if input_signed:
+            input_signed = model.get_layer(0).parameters.input_signed
+        type = 'int8' if input_signed else 'uint8'
+        pots_batch = model.predict(batch.astype(type))
 
         if labels is None:
             labels = label_batch
@@ -174,7 +151,7 @@ def evaluate_akida_model(model, x, activation):
 
 
 def get_training_parser(batch_size,
-                        calibrate=False,
+                        extract=False,
                         tune=False,
                         freeze_before=False,
                         global_batch_size=True,
@@ -186,7 +163,7 @@ def get_training_parser(batch_size,
 
     Args:
         batch_size (int): batch size default value
-        calibrate (bool, optional): either to add a calibrate action or not. Defaults to
+        extract (bool, optional): either to add an extract action or not. Defaults to
             False.
         tune (bool, optional): either to add a tune action or not. Defaults to
             False.
@@ -201,7 +178,7 @@ def get_training_parser(batch_size,
     Returns:
         argparse.ArgumentParser, argparse.ArgumentParser, argparse.ArgumentParser,
         argparse.ArgumentParser, argparse.ArgumentParser, argparse._SubParsersAction: main parser
-        and train, calibrate, tune and eval actions subparsers and the subparser object.
+        and train, tune, eval, and extract actions subparsers and the subparser object.
     """
 
     # Define a strictly positive int type for parameters
@@ -274,16 +251,19 @@ def get_training_parser(batch_size,
                                             help="Tune a Keras model")
 
     # Other subparsers
-    calibrate_parser = None
-    if calibrate:
-        calibrate_parser = subparsers.add_parser("calibrate",
-                                                 parents=[parent_parser],
-                                                 help="Calibrate a Keras model")
-        calibrate_parser.add_argument("-ns",
-                                      "--num_samples",
-                                      type=positive_int,
-                                      default=1024,
-                                      help="Number of samples to use for calibration")
+    extract_parser = None
+    if extract:
+        extract_parser = subparsers.add_parser("extract",
+                                               parents=[global_parser],
+                                               help="Extract samples from the dataset")
+        if not global_batch_size:
+            extract_parser.add_argument("-b",
+                                        "--batch_size",
+                                        type=positive_int,
+                                        default=batch_size,
+                                        help="The batch size")
+        extract_parser.add_argument("-s", "--savefile", type=str,
+                                    default='samples.npz', help="Samples filename.")
 
     eval_parser = subparsers.add_parser("eval",
                                         help="Evaluate a model",
@@ -294,7 +274,7 @@ def get_training_parser(batch_size,
                              action='store_true',
                              help="Converts to an Akida model and evaluate")
 
-    return parser, train_parser, tune_parser, eval_parser, calibrate_parser, subparsers
+    return parser, train_parser, tune_parser, eval_parser, extract_parser, subparsers
 
 
 def print_history_stats(history):
@@ -328,6 +308,7 @@ class RestoreBest(ModelCheckpoint):
             save model is made based on either the maximization or the minimization of the monitored
             quantity. Defaults to 'max'.
     """
+
     def __init__(self, model, monitor='val_accuracy', mode='max'):
         self.model = model
         self.temp_dir = tempfile.TemporaryDirectory()
