@@ -19,6 +19,10 @@ ViT model definition.
 Inspired from https://github.com/faustomorales/vit-keras/blob/master/vit_keras/vit.py.
 """
 
+__all__ = ["vit_imagenet", "vit_ti16", "bc_vit_ti16", "bc_vit_ti16_imagenet_pretrained", "vit_s16",
+           "vit_s32", "vit_b16", "vit_b32", "vit_l16", "vit_l32", "CONFIG_TI", "CONFIG_S",
+           "CONFIG_B", "CONFIG_L", "apply_embedding_weights"]
+
 import keras
 import warnings
 import numpy as np
@@ -26,12 +30,11 @@ import scipy as sp
 import typing_extensions as tx
 
 from quantizeml.layers import AddPositionEmbs, ClassToken, ExtractToken
-from quantizeml.models import load_model
+from quantizeml import load_model
 
 from ..layer_blocks import transformer_block, norm_to_layer
 from ..utils import fetch_file
-
-BASE_WEIGHT_PATH = 'http://data.brainchip.com/models/AkidaV2/vit/'
+from ..model_io import get_model_path
 
 
 ConfigDict = tx.TypedDict(
@@ -129,7 +132,17 @@ def vit_imagenet(input_shape,
                  last_norm='LN',
                  softmax='softmax',
                  act="GeLU"):
-    """Build a ViT model.
+    """Instantiates the ViT architecture.
+
+    The Vision Transformer (ViT) is a model for image classification that employs a Transformer-like
+    architecture over patches of the image. An image is split into fixed-size patches, each of them
+    are then linearly embedded, position embeddings are added, and the resulting sequence of vectors
+    are fed to a standard Transformer encoder.
+
+    Please refer to https://arxiv.org/abs/2010.11929 for further details.
+
+    Note: input preprocessing is included as part of the model (as a Rescaling layer). This model
+    expects inputs to be float tensors of pixels with values in the [0, 255] range.
 
     Args:
         input_shape (tuple): image shape tuple
@@ -209,7 +222,8 @@ def vit_ti16(input_shape=(224, 224, 3),
              softmax='softmax',
              act='GeLU',
              include_top=True):
-    """Build ViT-Tiny. All arguments passed to vit_imagenet.
+    """Instantiates the ViT-Tiny 16 architecture; that is a ViT architecture with 3 attention heads,
+    12 blocks and a patch size of 16.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
@@ -244,18 +258,27 @@ def vit_ti16(input_shape=(224, 224, 3),
     )
 
 
-def bc_vit_ti16(input_shape=(224, 224, 3), classes=1000, include_top=True):
-    """Build ViT-Tiny, changing all LN by LMN, using softmax2 and ReLU8.
+def bc_vit_ti16(input_shape=(224, 224, 3), classes=1000, include_top=True, num_blocks=12):
+    """Instantiates the ViT-Tiny 16 architecture adapted for implementation on hardware,
+    that is:
+
+        - LayerNormalization replaced by LayerMadNormalization,
+        - GeLU replaced by ReLU8 activations,
+        - Softmax replaced by shiftmax.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
         classes (int, optional): number of classes. Defaults to 1000.
         include_top (bool, optional): whether to include the final classifier network.
             Defaults to True.
+        num_blocks (int, optional): the number of transformer blocks to use. Defaults to 12.
 
     Returns:
         keras.Model: the requested model
     """
+    config_ti = CONFIG_TI.copy()
+    config_ti["num_blocks"] = num_blocks
+
     return vit_imagenet(
         name="vit-tiny",
         patch_size=16,
@@ -266,20 +289,37 @@ def bc_vit_ti16(input_shape=(224, 224, 3), classes=1000, include_top=True):
         softmax="softmax2",
         act="ReLU8",
         include_top=include_top,
-        **CONFIG_TI
+        **config_ti
     )
 
 
-def bc_vit_ti16_imagenet_pretrained():
-    """ Helper method to retrieve a `bc_vit_ti16` model that was trained on ImageNet dataset.
+def bc_vit_ti16_imagenet_pretrained(quantized=True):
+    """ Helper method to retrieve a ViT-Tiny 16 model adapted for implementation on hardware,
+    that is:
+
+        - LayerNormalization replaced by LayerMadNormalization,
+        - GeLU replaced by ReLU8 activations,
+        - Softmax replaced by shiftmax,
+
+    and that was trained on ImageNet dataset.
+
+    Args:
+        quantized (bool, optional): a boolean indicating whether the model should be loaded
+            quantized or not. Defaults to True.
 
     Returns:
-        keras.Model, dict: a Keras Model instance and the quantization configuration as a JSON
-        formatted dict.
+        keras.Model: a Keras Model instance
     """
-    model_name = 'bc_vit_ti16_224_i8_w4_a4.h5'
-    file_hash = '1c2002c4cbc999290a12b25eeb4dd59660b0fd317082b395bd7116e764209f50'
-    model_path = fetch_file(BASE_WEIGHT_PATH + model_name,
+    if quantized:
+        model_name_v2 = 'bc_vit_ti16_224_i8_w8_a8.h5'
+        file_hash_v2 = 'b147503b81991a5fbfabdd704fd17eb94fadb9faac1bb664bd195a16292d2e7f'
+    else:
+        model_name_v2 = 'bc_vit_ti16_224.h5'
+        file_hash_v2 = '7ae299f698abc545e21338b129dfbc0ff97ee9b994319e53e8605d34e3ee1f82'
+
+    model_path, model_name, file_hash = get_model_path("vit", model_name_v2=model_name_v2,
+                                                       file_hash_v2=file_hash_v2)
+    model_path = fetch_file(model_path,
                             fname=model_name,
                             file_hash=file_hash,
                             cache_subdir='models')
@@ -287,7 +327,8 @@ def bc_vit_ti16_imagenet_pretrained():
 
 
 def vit_s16(input_shape=(224, 224, 3), classes=1000, include_top=True):
-    """Build ViT-Small.
+    """Instantiates the ViT-Small 16 architecture; that is a ViT architecture with 6 attention heads,
+    12 blocks and a patch size of 16.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
@@ -309,7 +350,8 @@ def vit_s16(input_shape=(224, 224, 3), classes=1000, include_top=True):
 
 
 def vit_s32(input_shape=(224, 224, 3), classes=1000, include_top=True):
-    """Build ViT-Small.
+    """Instantiates the ViT-Small 32 architecture; that is a ViT architecture with 6 attention heads,
+    12 blocks and a patch size of 32.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
@@ -331,7 +373,8 @@ def vit_s32(input_shape=(224, 224, 3), classes=1000, include_top=True):
 
 
 def vit_b16(input_shape=(224, 224, 3), classes=1000, include_top=True):
-    """Build ViT-B16.
+    """Instantiates the ViT-B16 architecture; that is a ViT architecture with 12 attention heads,
+    12 blocks and a patch size of 16.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
@@ -353,7 +396,8 @@ def vit_b16(input_shape=(224, 224, 3), classes=1000, include_top=True):
 
 
 def vit_b32(input_shape=(224, 224, 3), classes=1000, include_top=True):
-    """Build ViT-B32.
+    """Instantiates the ViT-B32 architecture; that is a ViT architecture with 12 attention heads,
+    12 blocks and a patch size of 32.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
@@ -375,7 +419,8 @@ def vit_b32(input_shape=(224, 224, 3), classes=1000, include_top=True):
 
 
 def vit_l16(input_shape=(384, 384, 3), classes=1000, include_top=True):
-    """Build ViT-L16.
+    """Instantiates the ViT-L16 architecture; that is a ViT architecture with 16 attention heads,
+    24 blocks and a patch size of 16.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (384, 384, 3).
@@ -397,7 +442,8 @@ def vit_l16(input_shape=(384, 384, 3), classes=1000, include_top=True):
 
 
 def vit_l32(input_shape=(384, 384, 3), classes=1000, include_top=True):
-    """Build ViT-L32.
+    """Instantiates the ViT-L32 architecture; that is a ViT architecture with 16 attention heads,
+    24 blocks and a patch size of 32.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (384, 384, 3).
