@@ -21,21 +21,15 @@ Utilities for akida_models package.
 import os
 import urllib
 import time
-import numpy as np
-import warnings
-from pathlib import Path
 
 from six.moves.urllib.parse import urlsplit
 
 import tensorflow as tf
-import tensorflow_addons as tfa
-from keras.utils import io_utils
-from keras.utils.data_utils import validate_file, _extract_archive
-from keras.utils.generic_utils import Progbar
+from keras.src.utils import io_utils
+from keras.src.utils.data_utils import validate_file, _extract_archive
+from keras.utils import Progbar
 from keras.callbacks import TensorBoard
-
-from cnn2snn import load_quantized_model
-from quantizeml.models import load_model as qml_load_model
+from cnn2snn import get_akida_version, AkidaVersion
 
 
 def fetch_file(origin, fname=None, file_hash=None, cache_subdir="datasets", extract=False,
@@ -57,7 +51,8 @@ def fetch_file(origin, fname=None, file_hash=None, cache_subdir="datasets", extr
             location. Defaults to 'datasets'.
         extract (bool, optional): True tries extracting the file as an Archive, like tar or zip.
             Defaults to False.
-        cache_dir (str, optional): location to store cached files, when None it defaults to the
+        cache_dir (str, optional): location to store cached files, when directory does not exist it
+            defaults to /tmp/.keras, when None it defaults to the
             default directory `~/.keras/`. Defaults to None.
 
     Returns:
@@ -114,14 +109,24 @@ def fetch_file(origin, fname=None, file_hash=None, cache_subdir="datasets", extr
                     self.progbar.update(self.progbar.target)
                     self.finished = True
 
-        error_msg = "URL fetch failure on {}: {} -- {}"
+        error_msg = "URL fetch failure on {} (attempt number {}). \nReason: {}"
+        tries = 3
         try:
-            try:
-                urllib.request.urlretrieve(origin, fpath, DLProgbar())
-            except urllib.error.HTTPError as e:
-                raise Exception(error_msg.format(origin, e.code, e.msg))
-            except urllib.error.URLError as e:
-                raise Exception(error_msg.format(origin, e.errno, e.reason))
+            for attempt in range(tries):
+                try:
+                    urllib.request.urlretrieve(origin, fpath, DLProgbar())
+                except (urllib.error.HTTPError, urllib.error.URLError, ConnectionResetError) as e:
+                    if attempt < tries - 1:
+                        io_utils.print_msg(f"Error downloading data from {origin} to {fpath}, "
+                                           "retrying...")
+                        continue
+                    raise Exception(error_msg.format(origin, attempt+1, str(e)))
+                else:
+                    io_utils.print_msg("Download complete.")
+                finally:
+                    if attempt != 0:
+                        io_utils.print_msg(f"Download failed {attempt} time(s).")
+                break
         except (Exception, KeyboardInterrupt):
             if os.path.exists(fpath):
                 os.remove(fpath)
@@ -133,25 +138,6 @@ def fetch_file(origin, fname=None, file_hash=None, cache_subdir="datasets", extr
     return fpath
 
 
-def load_model(model_path):
-    """Combine the cnn2snn.load_quantized_model and quantizeml.load_model
-
-    Args:
-        model_path (str): model path
-
-    Returns:
-        keras.Model: the load model
-    """
-    try:
-        model = load_quantized_model(model_path)
-    except Exception:
-        try:
-            model = qml_load_model(model_path, custom_layers={'GELU': tfa.layers.GELU})
-        except Exception as e:
-            raise e.__class__('Cannot load provided model.')
-    return model
-
-
 def get_tensorboard_callback(out_dir, histogram_freq=1, prefix=''):
     """Build a Tensorboard call, pointing to the output directory
 
@@ -184,68 +170,20 @@ def get_tensorboard_callback(out_dir, histogram_freq=1, prefix=''):
                        profile_batch=0)
 
 
-def apply_weights_to_model(model, weights, verbose=True):
-    """Loads weights from a dictionary and apply it to a model.
+def get_params_by_version(relu_v2='ReLU3.75'):
+    """Provides the layer parameters depending on Akida version
 
-    Go through the dictionary of weights, find the corresponding variable
-    in the model and partially load its weights.
+    With Akida v1, sepconv are fused, the ReLU max value is 6.
+    With Akida v2, sepconv are unfused, the ReLU max value is "relu_v2" and the ReLU is at the end
+    of the block with GAP.
 
     Args:
-        model (keras.Model): the model to update
-        weights (dict): the dictionary of weights
-        verbose (bool, optional): if True, throw warning messages if a dict
-        item is not found in the model. Defaults to True.
+        relu_v2 (str, optional): ReLUx string when targetting V2. Defaults to ReLU3.75.
+    Returns:
+        bool, bool, str: fused, post_relu_gap, relu_activation
     """
-    if len(weights) == 0:
-        warnings.warn("There is no weight to apply to the model.")
-        return
-
-    # Go through the dictionary of weights with each item
-    for key, value in weights.items():
-        value_applied = False
-        for dest_var in model.variables:
-            if key == dest_var.name:
-                # Apply the current item value
-                dest_var.assign(value)
-                value_applied = True
-                break
-        if not value_applied and verbose:
-            warnings.warn(f"Variable '{key}' not found in the model.")
-
-
-def load_weights(model, weights_path):
-    """Loads weights from a npz file and apply it to a model.
-
-    Go through the dictionary of weights of the npz file, find the
-    corresponding variable in the model and partially load its weights.
-
-    Args:
-        model (keras.Model): the model to update
-        weights_path (str): the path of the npz file to load
-    """
-    # Check the npz file validity
-    path = Path(weights_path)
-    if not path.is_file():
-        raise ValueError(f"File `{weights_path}` not found.")
-
-    # Open the npz file
-    weights_dict = np.load(weights_path)
-
-    # Apply the weights to the model
-    apply_weights_to_model(model, weights_dict)
-
-
-def save_weights(model, weights_path):
-    """Save model weights on an npz file.
-
-    Takes a model and save the weights of all its layers into an npz file.
-
-    Args:
-        model (keras.Model): the model to save its weights
-        weights_path (str): the path of the npz file to save
-    """
-    weights_dict = {}
-    for var in model.variables:
-        weights_dict[var.name] = var
-
-    np.savez(weights_path, **weights_dict)
+    # Model version management
+    if get_akida_version() == AkidaVersion.v1:
+        return True, False, 'ReLU6'
+    # Akida v2
+    return False, True, relu_v2
