@@ -18,55 +18,32 @@
 VGG model definition for UTKFace regression.
 """
 
-from keras import Model
-from keras.layers import Dropout, Flatten, Input, Rescaling
+__all__ = ["vgg_utk_face", "vgg_utk_face_pretrained"]
 
-from cnn2snn import load_quantized_model, quantize
+from keras import Model
+from keras.layers import Dropout, Input, Rescaling
 
 from ..layer_blocks import conv_block, dense_block
-from ..utils import fetch_file
-
-BASE_WEIGHT_PATH = 'http://data.brainchip.com/models/AkidaV1/vgg/'
+from ..utils import fetch_file, get_params_by_version
+from ..model_io import load_model, get_model_path
 
 
-def vgg_utk_face(input_shape=(32, 32, 3),
-                 weight_quantization=0,
-                 activ_quantization=0,
-                 input_weight_quantization=None,
-                 input_scaling=(127, -1)):
+def vgg_utk_face(input_shape=(32, 32, 3), input_scaling=(127, -1)):
     """Instantiates a VGG-like model for the regression example on age
     estimation using UTKFace dataset.
 
+    Note: input preprocessing is included as part of the model (as a Rescaling layer). This model
+    expects inputs to be float tensors of pixels with values in the [0, 255] range.
+
     Args:
-        input_shape (tuple): input shape tuple of the model
-        weight_quantization (int): sets all weights in the model to have
-            a particular quantization bitwidth except for the weights in the
-            first layer.
-
-            * '0' implements floating point 32-bit weights.
-            * '2' through '8' implements n-bit weights where n is from 2-8 bits.
-        activ_quantization (int): sets all activations in the model to have a
-            particular activation quantization bitwidth.
-
-            * '0' implements floating point 32-bit activations.
-            * '1' through '8' implements n-bit weights where n is from 1-8 bits.
-        input_weight_quantization (int): sets weight quantization in the first
-            layer. Defaults to weight_quantization value.
-
-            * 'None' implements the same bitwidth as the other weights.
-            * '0' implements floating point 32-bit weights.
-            * '2' through '8' implements n-bit weights where n is from 2-8 bits.
+        input_shape (tuple, optional): input shape tuple of the model. Defaults to (32, 32, 3).
         input_scaling (tuple, optional): scale factor and offset to apply to
             inputs. Defaults to (127, -1). Note that following Akida convention,
-            the scale factor is an integer used as a divider.
+            the scale factor is an integer used as a divisor.
 
     Returns:
         keras.Model: a Keras model for VGG/UTKFace
     """
-    # Overrides input weight quantization if None
-    if input_weight_quantization is None:
-        input_weight_quantization = weight_quantization
-
     img_input = Input(shape=input_shape, name="input")
 
     if input_scaling is None:
@@ -75,13 +52,16 @@ def vgg_utk_face(input_shape=(32, 32, 3),
         scale, offset = input_scaling
         x = Rescaling(1. / scale, offset, name="rescaling")(img_input)
 
+    # Model version management
+    _, post_relu_gap, relu_activation = get_params_by_version()
+
     x = conv_block(x,
                    filters=32,
                    kernel_size=(3, 3),
                    name='conv_0',
                    use_bias=False,
-                   add_batchnorm=True,
-                   add_activation=True)
+                   relu_activation=relu_activation,
+                   add_batchnorm=True)
 
     x = conv_block(x,
                    filters=32,
@@ -91,8 +71,8 @@ def vgg_utk_face(input_shape=(32, 32, 3),
                    pooling='max',
                    pool_size=2,
                    use_bias=False,
-                   add_batchnorm=True,
-                   add_activation=True)
+                   relu_activation=relu_activation,
+                   add_batchnorm=True)
 
     x = Dropout(0.3, name="dropout_3")(x)
 
@@ -102,8 +82,8 @@ def vgg_utk_face(input_shape=(32, 32, 3),
                    padding='same',
                    name='conv_2',
                    use_bias=False,
-                   add_batchnorm=True,
-                   add_activation=True)
+                   relu_activation=relu_activation,
+                   add_batchnorm=True)
 
     x = conv_block(x,
                    filters=64,
@@ -113,8 +93,8 @@ def vgg_utk_face(input_shape=(32, 32, 3),
                    pooling='max',
                    pool_size=2,
                    use_bias=False,
-                   add_batchnorm=True,
-                   add_activation=True)
+                   relu_activation=relu_activation,
+                   add_batchnorm=True)
 
     x = Dropout(0.3, name="dropout_4")(x)
 
@@ -124,43 +104,53 @@ def vgg_utk_face(input_shape=(32, 32, 3),
                    padding='same',
                    name='conv_4',
                    use_bias=False,
-                   add_batchnorm=True,
-                   add_activation=True)
+                   relu_activation=relu_activation,
+                   pooling='global_avg',
+                   post_relu_gap=post_relu_gap,
+                   add_batchnorm=True)
 
     x = Dropout(0.3, name="dropout_5")(x)
-    x = Flatten(name="flatten")(x)
 
     x = dense_block(x,
                     units=64,
                     name='dense_1',
                     use_bias=False,
-                    add_batchnorm=True,
-                    add_activation=True)
+                    relu_activation=relu_activation,
+                    add_batchnorm=True)
 
-    x = dense_block(x, units=1, name='dense_2', add_activation=False)
+    x = dense_block(x, units=1, name='dense_2', relu_activation=False)
 
-    model = Model(img_input, x, name='vgg_utk_face')
-
-    if ((weight_quantization != 0) or (activ_quantization != 0) or
-            (input_weight_quantization != 0)):
-        return quantize(model, weight_quantization, activ_quantization,
-                        input_weight_quantization)
-    return model
+    return Model(img_input, x, name='vgg_utk_face')
 
 
-def vgg_utk_face_pretrained():
+def vgg_utk_face_pretrained(quantized=True):
     """
     Helper method to retrieve a `vgg_utk_face` model that was trained on
     UTK Face dataset.
 
+    Args:
+        quantized (bool, optional): a boolean indicating whether the model should be loaded
+            quantized or not. Defaults to True.
+
     Returns:
         keras.Model: a Keras Model instance.
 
     """
-    model_name = 'vgg_utk_face_iq8_wq2_aq2.h5'
-    file_hash = 'e341d2d5e4655846ddc7aceff0d4e324cbfbcca16f3cfefc65e7b0863e4a23a3'
-    model_path = fetch_file(BASE_WEIGHT_PATH + model_name,
+    if quantized:
+        model_name_v1 = 'vgg_utk_face_iq8_wq2_aq2.h5'
+        file_hash_v1 = 'e341d2d5e4655846ddc7aceff0d4e324cbfbcca16f3cfefc65e7b0863e4a23a3'
+        model_name_v2 = 'vgg_utk_face_i8_w4_a4.h5'
+        file_hash_v2 = '06036da929d1606ed38ec7c76cf80f8e784cebcd3e5340dfcd7f25705f3c0bc2'
+    else:
+        model_name_v1 = 'vgg_utk_face.h5'
+        file_hash_v1 = '3f9084c452ef006a16f10f2d124804b6db267f7a5149b735a9c383a9b00a4922'
+        model_name_v2 = 'vgg_utk_face.h5'
+        file_hash_v2 = 'f1fc29e4ab0c1615902d0dd48afc6bad1f3bbf4b1a6a2e3642e5c2ee37208121'
+
+    model_path, model_name, file_hash = get_model_path("vgg", model_name_v1, file_hash_v1,
+                                                       model_name_v2, file_hash_v2)
+    model_path = fetch_file(model_path,
                             fname=model_name,
                             file_hash=file_hash,
                             cache_subdir='models')
-    return load_quantized_model(model_path)
+    return load_model(model_path)
