@@ -22,10 +22,12 @@ import numpy as np
 
 from keras.callbacks import LearningRateScheduler
 
-from cnn2snn import load_quantized_model, convert
+from cnn2snn import convert
 
 from ..training import (get_training_parser, compile_model, evaluate_model, print_history_stats,
                         save_model)
+from ..extract import extract_samples
+from ..model_io import load_model
 from .preprocessing import load_data
 
 
@@ -33,7 +35,7 @@ def get_data():
     """ Loads UTKFace data.
 
     Returns:
-        np.array, np.array, np.array, np.array:  train set, train labels, test
+        np.array, np.array, np.array, np.array: train set, train labels, test
             set and test labels
     """
     # Load the dataset
@@ -51,7 +53,7 @@ def train_model(model, x_train, y_train, x_test, y_test, epochs, batch_size):
         y_train (numpy.ndarray): train labels
         x_test (numpy.ndarray): test data
         y_test (numpy.ndarray): test labels
-        epochs (int):  the number of epochs
+        epochs (int): the number of epochs
         batch_size (int): the batch size
     """
     # Learning rate: be more aggressive at the beginning, and apply decay
@@ -75,14 +77,14 @@ def train_model(model, x_train, y_train, x_test, y_test, epochs, batch_size):
 def main():
     """ Entry point for script and CLI usage.
     """
-    parser = get_training_parser(batch_size=128, global_batch_size=False)[0]
+    parser = get_training_parser(batch_size=128, global_batch_size=False, extract=True)[0]
     args = parser.parse_args()
 
     # Load the source model
-    model = load_quantized_model(args.model)
+    model = load_model(args.model)
 
     # Compile model
-    compile_model(model, loss='mae', metrics=None)
+    compile_model(model, loss='mae', metrics=['mae'])
 
     # Load data
     x_train, y_train, x_test, y_test = get_data()
@@ -103,6 +105,9 @@ def main():
             print("Validation accuracy: {0:.4f}".format(mae))
         else:
             evaluate_model(model, x_test, y=y_test, print_history=True)
+    elif args.action == 'extract':
+        # Extract samples from dataset
+        extract_samples(args.savefile, x_train, args.batch_size)
 
 
 if __name__ == "__main__":
